import openpyxl
from difflib import SequenceMatcher
import os


def read_workload(path: str):
    wb = openpyxl.load_workbook(path)
    ws = wb.active

    data = [row for row in ws.values]
    header = data[0]
    data = data[1:]

    wb.close()

    REQUEST_ID = 0
    OPERATION_ID = 4
    REQUEST = 5
    STATUS_CODE = 6
    RESPONSE = 9

    # api = {opId: [(reqId, req, statusCode, resp)]}
    api = {}
    for row in data:
        reqId = row[REQUEST_ID]
        opId = row[OPERATION_ID]
        req = row[REQUEST]
        statusCode = row[STATUS_CODE]
        resp = row[RESPONSE]

        if opId not in api:
            api[opId] = []

        tup = (reqId, req, statusCode, resp)
        api[opId].append(tup)

    return api


def read_faultload(path: str):
    wb = openpyxl.load_workbook(path)
    ws = wb.active

    data = [row for row in ws.values]
    header = data[0]
    data = data[1:]

    wb.close()

    REQUEST_ID = 0
    OPERATION_ID = 4
    REQUEST = 5
    LOCATION = 6
    PARAMETER = 7
    FAULT = 8
    FAULTY_INPUT = 10
    STATUS_CODE = 11
    RESPONSE = 14

    # api = {opId: {"param (loc)": {fault: [(reqId, faultyInput, req, statusCode, resp)]}}}
    api = {}
    for row in data:
        reqId = row[REQUEST_ID]
        opId = row[OPERATION_ID]
        req = row[REQUEST]
        loc = row[LOCATION]
        param = row[PARAMETER]
        fault = row[FAULT]
        faultyInput = row[FAULTY_INPUT]
        statusCode = row[STATUS_CODE]
        resp = row[RESPONSE]

        if fault in ["No-op fault", "Replace with predefined string"]:
            print("Found '" + fault + "' - ignoring")
            continue

        if opId not in api:
            api[opId] = {}
        paramLoc = param + " (" + loc + ")"
        if paramLoc not in api[opId]:
            api[opId][paramLoc] = {}
        faultVerified = rectify_fault(fault, faultyInput)
        if faultVerified not in api[opId][paramLoc]:
            api[opId][paramLoc][faultVerified] = []

        tup = (reqId, faultyInput, req, statusCode, resp)
        api[opId][paramLoc][faultVerified].append(tup)

    return api


def rectify_fault(fault: str, faultyInput: str):
    if fault == "Replace with malicious string":
        if faultyInput == "\" or 1=0 −":
            return "Append  or 1=0 --"
        elif faultyInput == "\" or 1=1 −":
            return "Append  or 1=1 --"
        elif faultyInput == "\" or 1=1 or \" \"=\"":
            return "Append  or 1=1 or ''='"
        elif faultyInput == "' UNION ALL SELECT":
            return "Append  UNION ALL SELECT"
        elif faultyInput == "' UNION SELECT":
            return "Append  UNION SELECT"
        elif faultyInput == "char%2839%29%2b%28SELECT":
            return "Append char%2839%29%2b%28SELECT"
        elif faultyInput == "char%4039%41%2b%40SELECT":
            return "Append char%4039%41%2b%40SELECT"
        elif faultyInput == "&quot; or 1=1 or &quot;&quot;=&quot;":
            return "Append &quot; or 1=1 or &quot;&quot;=&quot;"
        elif faultyInput == "&apos; or &apos;&apos;=&apos;":
            return "Append &apos; or &apos;&apos;=&apos;"
        elif faultyInput.endswith("or 1=0 --") or faultyInput.endswith("or (1=0 --"):
            return "Append  or 1=0 --"
        elif faultyInput.endswith("or 1=1 --"):
            return "Append  or 1=1 --"
        elif faultyInput.endswith("or 1=1 or ''=") or faultyInput.endswith("or 1=1 or \"\"=\""):
            return "Append  or 1=1 or ''='"
        elif faultyInput.endswith("admin' --") or faultyInput.endswith("admin\" --"):
            return "Append  admin' --"
        elif faultyInput.endswith("UNION ALL SELECT"):
            return "Append  UNION ALL SELECT"
        elif faultyInput.endswith("UNION SELECT"):
            return "Append  UNION SELECT"
        elif faultyInput.endswith("char%2839%29%2b%28SELECT"):
            return "Append char%2839%29%2b%28SELECT"
        elif faultyInput.endswith("char%4039%41%2b%40SELECT"):
            return "Append char%4039%41%2b%40SELECT"
        elif faultyInput.endswith("&quot; or 1=1 or &quot;&quot;=&quot;"):
            return "Append &quot; or 1=1 or &quot;&quot;=&quot;"
        elif faultyInput.endswith("&apos; or &apos;&apos;=&apos;"):
            return "Append &apos; or &apos;&apos;=&apos;"
        else:
            return "Append one of 801 SQL injection strings covering different versions of multiple Database Systems"
    elif fault == "Replace with invalid date":
        if "1984-2-29" in faultyInput:
            return "Replace with invalid date 1985-2-29"
        elif "1998-4-31" in faultyInput:
            return "Replace with invalid date 1998-4-31"
        elif "1997-13-1" in faultyInput:
            return "Replace with invalid date 1997-13-1"
        elif "1994-12-0" in faultyInput:
            return "Replace with invalid date 1994-12-0"
        elif "1992-9-31" in faultyInput:
            return "Replace with invalid date 1992-9-31"
        elif "1993-8-32" in faultyInput:
            return "Replace with invalid date 1993-8-32"
    elif fault == "Replace with invalid time":
        if "13:00:61" in faultyInput:
            return "Replace with invalid time 13:00:61"
        elif "10:73:02" in faultyInput:
            return "Replace with invalid time 10:73:02"
        elif "25:58:04" in faultyInput:
            return "Replace with invalid time 25:58:04"
        elif "04:03:60" in faultyInput:
            return "Replace with invalid time 04:03:60"
        elif "07:60:15" in faultyInput:
            return "Replace with invalid time 07:60:15"
        elif "24:05:01" in faultyInput:
            return "Replace with invalid time 24:05:01"

    if fault in ["Replace with malicious string", "Replace with invalid date", "Replace with invalid time"]:
        raise Exception("Could not find a fitting fault for input:" + faultyInput + " (original fault: " + fault + ")")

    return fault


# workload    api = {opId: [(reqId, req, statusCode, resp)]}
# faultload  api = {opId: {"param (loc)": {fault: [(reqId, faultyInput, req, statusCode, resp)]}}}
def print_api(api: dict):
    print("API")
    for opId in api:
        print("\t" + opId)
        if type(api[opId]) is list:
            for tup in api[opId]:
                print("\t\t(reqId:" + tup[0] + ", req, statusCode:" + tup[2] + ", resp)")
        elif type(api[opId]) is dict:
            for param in api[opId]:
                print("\t\t" + param)
                for fault in api[opId][param]:
                    c = len(api[opId][param][fault])
                    print("\t\t\t" + fault + " (" + str(c) + ")")
                    for tup in api[opId][param][fault]:
                        print("\t\t\t\t(reqId:" + tup[0] + ", faultyInput, req, statusCode:" + tup[3] + ", resp)")


# workload    api = {opId: [(reqId, req, statusCode, resp)]}
# faultload  api = {opId: {"param (loc)": {fault: [(reqId, faultyInput, req, statusCode, resp)]}}}
def response_similarity(api_name: str, api_workload: dict, api_faultload: dict, same_file: bool = False):
    if os.path.exists("Response similarity.xlsx") and same_file:
        wb = openpyxl.load_workbook("Response similarity.xlsx")
        ws = wb.create_sheet(api_name)
    else:
        wb = openpyxl.Workbook()
        ws = wb.active
        ws.title = api_name
    header = ["Operation ID", "Parameter", "Fault", "Request ID", "Request", "Faulty input", "WL Status code",
              "FL Status code", "Status match?", "WL Response", "FL Response", "Response similarity"]
    c = 0
    r = 1
    for h in header:
        c += 1
        ws.cell(r, c, h)
    for opId in api_faultload:
        for param in api_faultload[opId]:
            for fault in api_faultload[opId][param]:
                for fl_tup in api_faultload[opId][param][fault]:
                    wl_tups = [wl_tup for wl_tup in api_workload[opId] if wl_tup[0] == fl_tup[0]]
                    for wl_tup in wl_tups:
                        r += 1
                        data = [opId, param, fault, fl_tup[0], fl_tup[2], fl_tup[1], wl_tup[2], fl_tup[3],
                                wl_tup[2] == fl_tup[3], wl_tup[3], fl_tup[4],
                                SequenceMatcher(a=wl_tup[3], b=fl_tup[4]).ratio()]
                        c = 0
                        for datum in data:
                            c += 1
                            ws.cell(r, c, datum)
    wb.save(("" if same_file else api_name + " - ") + "Response similarity.xlsx")
    wb.close()


# workload    api = {opId: [(reqId, req, statusCode, resp)]}
# faultload  api = {opId: {"param (loc)": {fault: [(reqId, faultyInput, req, statusCode, resp)]}}}
def response_sequence(api_name: str, api_faultload: dict, same_file: bool = False):
    if os.path.exists("Response sequence.xlsx") and same_file:
        wb = openpyxl.load_workbook("Response sequence.xlsx")
        ws = wb.create_sheet(api_name)
    else:
        wb = openpyxl.Workbook()
        ws = wb.active
        ws.title = api_name
    header = ["Operation ID", "Parameter", "Fault", "Request IDs", "Status codes",
              "Status match?", "Response similarity"]
    c = 0
    r = 1
    for h in header:
        c += 1
        ws.cell(r, c, h)
    for opId in api_faultload:
        for param in api_faultload[opId]:
            for fault in api_faultload[opId][param]:
                tups = api_faultload[opId][param][fault]
                req_ids = str([tup[0] for tup in tups])
                status_codes = [tup[3] for tup in tups]
                equal = all([status_codes[0] == s for s in status_codes])
                responses = [tup[4] for tup in tups]
                similarity = [round(SequenceMatcher(a=responses[0], b=r).ratio(), 3) for r in responses]

                r += 1
                data = [opId, param, fault, str(req_ids).replace("'", ""), str(status_codes).replace("'", ""),
                        equal, str(similarity)]
                c = 0
                for datum in data:
                    c += 1
                    ws.cell(r, c, datum)
    wb.save(("" if same_file else api_name + " - ") + "Response sequence.xlsx")
    wb.close()


def main():
    wl_apis = ["Ably", "Api2Pdf", "AppVeyor", "AucklandMuseum", "BikeWise", "Browshot", "BulkSMSJSON", "Calendar",
                 "CallControl", "CallFire", "CenitIO", "CollegeFootballData", "CrossBrowserTesting", "D7SMS",
                 "DockerEngineAPI", "Drive", "ElmahIo", "Europeana1", "Europeana2", "Figshare", "Giphy", "GreenwirePublic",
                 "HHSMediaServices", "HighwaysEnglandAPI", "LaunchDarkly", "Miataru", "MoonByAiWeiweiOlafurEliasson",
                 "NeoWS", "NSIDC", "ODWeather", "OpenSkills", "RatGenomeDatabase", "Slack", "Spotify", "TD", "Traccar",
                 "Trello", "USEPAECHO_AllData", "USEPAECHO_DFR", "USEPAECHO_ECR", "USEPAECHO_RCRA", "USEPAECHO_SDWA",
                 "WavecellSmsApi", "WeGA", "WhoHostsThis", "WMATABusRoute", "WMATARailStation", "XKCD", "ZoomConnect"]
    wl_apis_lite = ["Ably", "Api2Pdf", "AppVeyor", "AucklandMuseum", "BikeWise", "Browshot", "BulkSMSJSON",
               "CallControl", "CallFire", "CollegeFootballData", "CrossBrowserTesting", "D7SMS",
               "DockerEngineAPI", "Drive", "Europeana1", "Europeana2", "Figshare",
               "GreenwirePublic",
               "HHSMediaServices", "HighwaysEnglandAPI",
               "NeoWS", "NSIDC", "ODWeather", "OpenSkills", "RatGenomeDatabase", "Slack", "Spotify", "TD", "Traccar",
               "WavecellSmsApi", "ZoomConnect"]

    # for api in wl_apis:
    #     print("[" + api + "]")
    #     api_workload = read_workload("..\\Results\\results - workload\\" + api + " - workload.xlsx")
    #     api_faultload = read_faultload("..\\Results\\results - faultload\\" + api + " - faultload.xlsx")
    #     response_similarity(api, api_workload, api_faultload)
    # for api in wl_apis:
    #     print("[" + api + "]")
    #     api_faultload = read_faultload("..\\Results\\results - faultload\\" + api + " - faultload.xlsx")
    #     response_sequence(api, api_faultload)


if __name__ == '__main__':
    main()
