import openpyxl
from difflib import SequenceMatcher
import os


def read_faultload(path: str):
    wb = openpyxl.load_workbook(path)
    ws = wb.active

    data = [row for row in ws.values]
    header = data[0]
    data = data[1:]

    wb.close()
    if "tpc-" in path.lower():
        OPERATION_ID = 2
        LOCATION = 5
        PARAMETER = 6
    else:
        OPERATION_ID = 4
        LOCATION = 6
        PARAMETER = 7

    # api = {opId: ["param (loc)"]}
    api = {}
    for row in data:
        opId = row[OPERATION_ID]
        loc = row[LOCATION]
        param = row[PARAMETER]

        if len(param) == 0:
            continue

        if opId not in api:
            api[opId] = set()
        api[opId].add(param + " (" + loc + ")")

    return api


def read_text(file_name: str, sep: str):

    content = []
    f = open(file_name, "r")
    for line in f:
        line_stripped = line.strip()
        if line_stripped and len(line_stripped.split(sep)) == 2:
            split = line_stripped.split(sep)
            tup = (split[0].strip(), split[1].strip())
            content.append(tup)
    f.close()
    return content


# api = {opId: ["param (loc)"]}
def count_parameters(api: dict, opIds: list):

    try:
        return len([param for opId in opIds for param in api[opId]])
    except:
        return "one of the operations was not found"


def main():
    apis = {"Ably": "Ably", "Api2Pdf": "Api2Pdf", "AppVeyor": "AppVeyor", "Auckland Museum": "AucklandMuseum",
            "BikeWise": "BikeWise", "Browshot": "Browshot", "BulkSMS": "BulkSMSJSON", "Google Calendar": "Calendar",
            "Call Control": "CallControl", "Callfire": "CallFire", "Cenit IO": "CenitIO",
            "College Football Data": "CollegeFootballData", "Cross Browser Testing": "CrossBrowserTesting",
            "D7SMS": "D7SMS", "Docker Engine API": "DockerEngineAPI", "Google Drive": "Drive", "elmah.io": "ElmahIo",
            "Europeana": "Europeana1", "Figshare": "Figshare", "Giphy": "Giphy", "Greenwire Public": "GreenwirePublic",
            "HHS Media Services": "HHSMediaServices", "Highways England": "HighwaysEnglandAPI",
            "LaunchDarkly": "LaunchDarkly", "Miataru": "Miataru", "Moon": "MoonByAiWeiweiOlafurEliasson",
            "NeoWS": "NeoWS", "NSIDC": "NSIDC", "ODWeather": "ODWeather", "Open Skills": "OpenSkills",
            "Rat Genome Database": "RatGenomeDatabase", "Slack": "Slack", "Spotify": "Spotify", "Kazoo Crossbar": "TD",
            "Traccar": "Traccar", "Trello": "Trello", "US EPA ECHO - All Data": "USEPAECHO_AllData",
            "US EPA ECHO - DFR": "USEPAECHO_DFR", "US EPA ECHO - ECR": "USEPAECHO_ECR",
            "US EPA ECHO - RCRA": "USEPAECHO_RCRA", "US EPA ECHO - SDWA": "USEPAECHO_SDWA",
            "Wavecell SMS": "WavecellSmsApi", "WeGA": "WeGA", "Who Hosts This": "WhoHostsThis",
            "WMATA Bus Route": "WMATABusRoute", "WMATA Rail Station": "WMATARailStation", "XKCD": "XKCD",
            "Zoom Connect": "ZoomConnect", "TPC-App": "TPC-App", "TPC-C": "TPC-C"}

    file = "Operations by service.txt"
    content = read_text(file, ":")
    for tup in content:
        api_name = tup[0]
        operations_str = tup[1]
        operation_list = [operation.strip() for operation in operations_str.split(",")]
        api_faultload = read_faultload("..\\Results\\results - faultload\\" + apis[api_name] + " - faultload.xlsx")
        param_count = count_parameters(api_faultload, operation_list)
        print("%s;%s;%s" % (api_name, operations_str, str(param_count)))


if __name__ == '__main__':
    main()
