import openpyxl
from difflib import SequenceMatcher
import os


def read_faultload(path: str):
    wb = openpyxl.load_workbook(path)
    ws = wb.active

    data = [row for row in ws.values]
    header = data[0]
    data = data[1:]

    wb.close()
    if "tpc-" in path.lower():
        OPERATION_ID = 2
        LOCATION = 5
        PARAMETER = 6
    else:
        OPERATION_ID = 4
        LOCATION = 6
        PARAMETER = 7

    # api = {opId: ["param (loc)"]}
    api = {}
    for row in data:
        opId = row[OPERATION_ID]
        loc = row[LOCATION]
        param = row[PARAMETER]

        if len(param) == 0:
            continue

        if opId not in api:
            api[opId] = set()
        api[opId].add(param + " (" + loc + ")")

    return api


# api = {opId: ["param (loc)"]}
def count_parameters(api: dict):

    return len([param for opId in api for param in api[opId]])


def main():
    apis = {"Ably": "Ably", "Api2Pdf": "Api2Pdf", "AppVeyor": "AppVeyor", "Auckland Museum": "AucklandMuseum",
            "BikeWise": "BikeWise", "Browshot": "Browshot", "BulkSMS": "BulkSMSJSON", "Google Calendar": "Calendar",
            "Call Control": "CallControl", "Callfire": "CallFire", "Cenit IO": "CenitIO",
            "College Football Data": "CollegeFootballData", "Cross Browser Testing": "CrossBrowserTesting",
            "D7SMS": "D7SMS", "Docker Engine API": "DockerEngineAPI", "Google Drive": "Drive", "elmah.io": "ElmahIo",
            "Europeana": "Europeana1", "Figshare": "Figshare", "Giphy": "Giphy", "Greenwire Public": "GreenwirePublic",
            "HHS Media Services": "HHSMediaServices", "Highways England": "HighwaysEnglandAPI",
            "LaunchDarkly": "LaunchDarkly", "Miataru": "Miataru", "Moon": "MoonByAiWeiweiOlafurEliasson",
            "NeoWS": "NeoWS", "NSIDC": "NSIDC", "ODWeather": "ODWeather", "Open Skills": "OpenSkills",
            "Rat Genome Database": "RatGenomeDatabase", "Slack": "Slack", "Spotify": "Spotify", "Kazoo Crossbar": "TD",
            "Traccar": "Traccar", "Trello": "Trello", "US EPA ECHO - All Data": "USEPAECHO_AllData",
            "US EPA ECHO - DFR": "USEPAECHO_DFR", "US EPA ECHO - ECR": "USEPAECHO_ECR",
            "US EPA ECHO - RCRA": "USEPAECHO_RCRA", "US EPA ECHO - SDWA": "USEPAECHO_SDWA",
            "Wavecell SMS": "WavecellSmsApi", "WeGA": "WeGA", "Who Hosts This": "WhoHostsThis",
            "WMATA Bus Route": "WMATABusRoute", "WMATA Rail Station": "WMATARailStation", "XKCD": "XKCD",
            "Zoom Connect": "ZoomConnect", "TPC-App": "TPC-App", "TPC-C": "TPC-C"}

    param_counts = []
    for api in apis:
        api_faultload = read_faultload("..\\Results\\results - faultload\\" + apis[api] + " - faultload.xlsx")
        param_count = count_parameters(api_faultload)
        print("%s: %d" % (api, param_count))
        param_counts.append(param_count)
    print("Total parameter count: %d" % sum(param_counts))


if __name__ == '__main__':
    main()
