library(readr)
library(ggplot2)
library(plyr)
library(scales)

Faults <- read_delim("D:/Documents/UC/2 Ano/Tese/4 - Evaluation/2 - Large/R/Faults ungrouped.csv", ";", escape_double = FALSE, col_types = cols(Count = col_integer()), trim_ws = TRUE)

real_fault_names = c("Replace with empty value", "Replace with null", "Duplicate random elements in the array",
                     "Remove random element from array", "Remove all elements in the array",
                     "Remove all elements in the array except the first one", "Negate boolean value",
                     "Overflow string representation of boolean value", "Add 1 unit", "Subtract 1 unit",
                     "Replace with a random positive value", "Replace with a random negative value", "Replace with 0",
                     "Replace with 1", "Replace with -1", "Replace with data type maximum", "Replace with data type minimum",
                     "Replace with data type maximum + 1", "Replace with data type minimum - 1",
                     "Replace with parameter domain minimum", "Replace with parameter domain minimum - 1",
                     "Replace with random printable character string of equal length",
                     "Replace with random alphanumeric string of equal length", "Append random non-printable characters",
                     "Insert random non-printable characters at random positions",
                     "Replace with random non-printable string of equal length", "Append SQL injection attack to original input",
                     "Add 100 years to the date", "Subtract 100 years from the date",
                     "Replace with last day of the previous millennium", "Replace with first day of the current millennium",
                     "Replace with invalid date 1985-2-29", "Replace with invalid date 1998-4-31",
                     "Replace with invalid date 1997-13-1", "Replace with invalid date 1994-12-0",
                     "Replace with invalid date 1993-8-32", "Add 24 hours to the time", "Subtract 24 hours from the time",
                     "Replace with invalid time 13:00:61", "Replace with invalid time 10:73:02", "Replace with invalid time 25:58:04",
                     "Replace with invalid time 04:03:60", "Replace with invalid time 07:60:15", "Replace with invalid time 24:05:01")

Faults$Fault = rep(real_fault_names, length(unique(Faults$Set)))

faultNames = unique(Faults$Fault)
for(i in 1:length(faultNames))
{
  s = sum(subset(Faults, Faults$Fault == faultNames[i])$Count)
  faultNames[i] = paste0(faultNames[i], " (", s, ")")
}

Faults$Fault = rep(faultNames, length(unique(Faults$Set)))


plot <- ggplot(Faults, aes(Fault, y=Count/sum(Count), fill = Set))
plot <- plot + geom_bar(stat = "identity", position = position_stack(reverse = TRUE))
#plot <- plot + scale_fill_brewer(palette="Set1", direction = -1)
plot <- plot + scale_fill_manual(values = c("#3e9651", "#ed7d31", "#777777", "#5b9bd5", "#cc2528")) #"#da7c30", "#3969b1", "#535154")) # "#6b4c9a"))
plot <- plot + coord_flip()
plot <- plot + scale_x_discrete(limits = rev(unique(Faults$Fault)))
plot <- plot + scale_y_continuous(labels = scales::percent_format(accuracy = 0.5, suffix = ""), breaks = seq(0, 10.5, 0.5)/100)
plot <- plot + theme(text = element_text(colour = "#000000", size = 15), axis.text=element_text(colour="#000000"),
                     legend.position = c(0.85, 0.75))
plot <- plot + xlab("Fault type") + ylab("Frequency (%)")
print(plot)




# fault_names = c("None", "Empty", "Array", "Boolean", "Date", "Numeric", "Malicious string", "Printable string", "Non-printable string")
# fault_count = c(7, 46, 9, 10, 2, 43, 57, 47, 38)
# 
# Faults = data.frame("Fault" = fault_names, "Frequency" = (fault_count/sum(fault_count)) * 100)
# 
# plot <- ggplot(Faults, aes(Fault, Frequency))
# plot <- plot + geom_bar(stat = "identity",  fill = "#396ab1")
# #plot <- plot + geom_text(stat='count', aes(label= round(((..count..)/sum(..count..)) * 100, 2)), position = position_dodge(width = 1), inherit.aes = TRUE)
# plot <- plot + coord_flip()
# plot <- plot + scale_x_discrete(limits = rev(fault_names))
# plot <- plot + scale_y_continuous(breaks = seq(0, 100, 2))
# plot <- plot + theme(text = element_text(colour = "#000000", size = 15), axis.text=element_text(colour="#000000"))
# plot <- plot + xlab("Fault type") + ylab("Frequency (%)")
# print(plot)
