import com.github.curiousoddman.rgxgen.RgxGen;
import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.parser.OpenAPIResolver;
import io.swagger.v3.parser.OpenAPIV3Parser;
import org.apache.commons.lang3.RandomUtils;
import org.jboss.resteasy.client.jaxrs.internal.ResteasyClientBuilderImpl;
import pt.uc.dei.rest_api_robustness_tester.request.Parameter;
import pt.uc.dei.rest_api_robustness_tester.security.AuthHandlers;
import pt.uc.dei.rest_api_robustness_tester.specification.*;
import pt.uc.dei.rest_api_robustness_tester.specification.RestApiSetup;
import pt.uc.dei.rest_api_robustness_tester.utils.Utils;
import pt.uc.dei.rest_api_robustness_tester.workload.ParameterFilter;

import javax.ws.rs.client.Client;
import javax.ws.rs.client.Entity;
import javax.ws.rs.client.WebTarget;
import javax.ws.rs.core.MediaType;
import java.io.File;
import java.util.ArrayList;
import java.util.List;

public class TalkdeskKazooCrossbar implements RestApiSetup
{
    @Override
    public RestApi Load()
    {
        Client client = new ResteasyClientBuilderImpl().disableTrustManager().build();
        WebTarget baseTarget = client.target("https://management-01.us-east-1.aws.tst.pub.talkdeskdev.com:8443/v2");
        
        String firstAuthToken = CreateApiKey(baseTarget);
        int amountToCreate = 20;
        List<String> accountIDs = CreateAccountIDs(baseTarget, firstAuthToken, amountToCreate);
        List<String> deviceIDs = new ArrayList<>();
        for(String accountID : accountIDs)
            deviceIDs.addAll(CreateDeviceIDs(baseTarget, firstAuthToken, accountID, amountToCreate));
        //List<String> phoneNumbers = new ArrayList<>();
        //for(String accountID : accountIDs)
        //    phoneNumbers.addAll(CreatePhoneNumbers(baseTarget, firstAuthToken, accountID, amountToCreate));
        client.close();
    
        File file = new File("API Specifications\\Companies\\Talkdesk", "Talkdesk - Kazoo Crossbar.yaml");
        AuthHandlers authHandlers = new AuthHandlers();
        authHandlers.AddAuthHandler("AUTH_TOKEN", reqBuilder -> reqBuilder.addHeader("X-Auth-Token", RefreshApiKey()));
        
        OpenAPI openAPI = new OpenAPIV3Parser().read(file.getAbsolutePath());
        OpenAPIResolver res = new OpenAPIResolver(openAPI);
        openAPI = res.resolve();
    
        RestApiConverter<OpenAPI> converter = new OpenApiToRestApi();
        RestApiSpecification restAPISpecification = converter.Convert(openAPI);
    
        RestApiConfig config = new RestApiConfig();
        config.AddParameterFilter(new ParameterFilter("ACCOUNT_ID", Parameter.Location.Path).
                    ApplyToAll().
                    SetValues(accountIDs)).
                AddParameterFilter(new ParameterFilter("DEVICE_ID", Parameter.Location.Path).
                    ApplyToAll().
                    SetValues(deviceIDs));
    
        return new RestApi("Talkdesk - Kazoo Crossbar", restAPISpecification, authHandlers, config);
    }
    
    private String CreateApiKey(WebTarget webTarget)
    {
        String jsonReq = "{\"data\": {\"credentials\": \"42ed84e7d274d76425ea79b2507d4388\", \"account_name\": \"talkconnect\"}}";
        
        String jsonResp = webTarget.path("user_auth").
                request(MediaType.APPLICATION_JSON).
                put(Entity.json(jsonReq)).
                readEntity(String.class);
    
        jsonResp = jsonResp.replace(" ", "");
        String startPattern = "\"auth_token\":\"";
        String endPattern = "\"";
        String authToken = jsonResp.substring(jsonResp.indexOf(startPattern) + startPattern.length());
        authToken = authToken.substring(0, authToken.indexOf(endPattern));
        System.out.println("Created Auth Token: " + authToken);
        return authToken;
    }
    
    private final int authDurationMillis = 30 * 60 * 1000;
    private long lastApiKeyMillis = System.currentTimeMillis();
    private String lastApiKey = null;

    private String RefreshApiKey()
    {
        if(lastApiKey == null || System.currentTimeMillis() >= lastApiKeyMillis + authDurationMillis)
        {
            Client client = new ResteasyClientBuilderImpl().disableTrustManager().build();
            WebTarget webTarget = client.target("https://management-01.us-east-1.aws.tst.pub.talkdeskdev.com:8443/v2");

            String jsonReq = "{\"data\": {\"credentials\": \"42ed84e7d274d76425ea79b2507d4388\", \"account_name\": \"talkconnect\"}}";

            String jsonResp = webTarget.path("user_auth").
                    request(MediaType.APPLICATION_JSON).
                    put(Entity.json(jsonReq)).
                    readEntity(String.class);
    
            client.close();

            lastApiKeyMillis = System.currentTimeMillis();

            jsonResp = jsonResp.replace(" ", "");
            String startPattern = "\"auth_token\":\"";
            String endPattern = "\"";
            String authToken = jsonResp.substring(jsonResp.indexOf(startPattern) + startPattern.length());
            authToken = authToken.substring(0, authToken.indexOf(endPattern));
            System.out.println("Created Auth Token: " + authToken);
            lastApiKey = authToken;
        }

        return lastApiKey;
    }
    
    private List<String> CreateAccountIDs(WebTarget webTarget, String authToken, int amount)
    {
        List<String> accountIDs = new ArrayList<>();
        
        for(int i = 0; i < amount; i++)
        {
            StringBuilder s = new StringBuilder();
            for (int j = 0; j < RandomUtils.nextInt(1, 62 + 1); j++)
                s.append(Utils.AsciiAlphanumericCharacters().
                        charAt(RandomUtils.nextInt(0, Utils.AsciiAlphanumericCharacters().length())));
    
            String jsonReq = "{\"data\": {\"name\": \"" + s.toString() + "\"}}";
    
            String jsonResp = webTarget.path("accounts").
                    request(MediaType.APPLICATION_JSON).
                    header("X-Auth-Token", authToken).
                    put(Entity.json(jsonReq)).
                    readEntity(String.class);
    
            jsonResp = jsonResp.replace(" ", "");
            String startPattern = "\"id\":\"";
            String endPattern = "\"";
            String accountID = jsonResp.substring(jsonResp.indexOf(startPattern) + startPattern.length());
            accountID = accountID.substring(0, accountID.indexOf(endPattern));
            accountIDs.add(accountID);
            System.out.println("Created Account ID: " + accountID);
        }
        
        return accountIDs;
    }
    
    private List<String> CreateDeviceIDs(WebTarget webTarget, String authToken, String accountID, int amount)
    {
        List<String> deviceIDs = new ArrayList<>();
    
        for(int i = 0; i < amount; i++)
        {
            StringBuilder s = new StringBuilder();
            for (int j = 0; j < RandomUtils.nextInt(1, 62 + 1); j++)
                s.append(Utils.AsciiAlphanumericCharacters().
                        charAt(RandomUtils.nextInt(0, Utils.AsciiAlphanumericCharacters().length())));
    
            String jsonReq = "{\"data\": {\"name\": \"" + s.toString() + "\"}}";
        
            String jsonResp = webTarget.path("accounts/" + accountID + "/devices").
                    request(MediaType.APPLICATION_JSON).
                    header("X-Auth-Token", authToken).
                    put(Entity.json(jsonReq)).
                    readEntity(String.class);
        
            jsonResp = jsonResp.replace(" ", "");
            String startPattern = "\"id\":\"";
            String endPattern = "\"";
            String deviceID = jsonResp.substring(jsonResp.indexOf(startPattern) + startPattern.length());
            deviceID = deviceID.substring(0, deviceID.indexOf(endPattern));
            deviceIDs.add(deviceID);
            System.out.println("[Account ID " + accountID + "] Created Device ID: " + deviceID);
        }
    
        return deviceIDs;
    }
    
    private List<String> CreatePhoneNumbers(WebTarget webTarget, String authToken, String accountID, int amount)
    {
        List<String> phoneNumbers = new ArrayList<>();
        for(int i = 0; i < amount; i++)
        {
            String E164phoneNumber = new RgxGen("^\\+?[0-9]{1,3}[1-9]\\d{1,14}$").generate();
            String jsonReq = "{\"data\": {\"my_own_field\": {}}}";
        
            String jsonResp = webTarget.path("accounts/" + accountID + "/phone_numbers/" + E164phoneNumber).
                    request(MediaType.APPLICATION_JSON).
                    header("X-Auth-Token", authToken).
                    put(Entity.json(jsonReq)).
                    readEntity(String.class);
        
            jsonResp = jsonResp.replace(" ", "");
            String startPattern = "\"id\":\"";
            String endPattern = "\"";
            String phoneNumber = jsonResp.substring(jsonResp.indexOf(startPattern) + startPattern.length());
            phoneNumber = phoneNumber.substring(0, phoneNumber.indexOf(endPattern));
            phoneNumbers.add(phoneNumber);
            System.out.println("[Account ID " + accountID + "] Created Phone Number: " + phoneNumber);
        }
    
        return phoneNumbers;
    }
}
