import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.parser.OpenAPIResolver;
import io.swagger.v3.parser.OpenAPIV3Parser;
import org.apache.commons.codec.binary.Base64;
import pt.uc.dei.rest_api_robustness_tester.Path;
import pt.uc.dei.rest_api_robustness_tester.request.Parameter;
import pt.uc.dei.rest_api_robustness_tester.specification.*;
import pt.uc.dei.rest_api_robustness_tester.specification.RestApiSetup;
import pt.uc.dei.rest_api_robustness_tester.workload.OperationFilter;
import pt.uc.dei.rest_api_robustness_tester.workload.ParameterFilter;
import pt.uc.dei.rest_api_robustness_tester.workload.PayloadFilter;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import static pt.uc.dei.rest_api_robustness_tester.Path.HttpMethod.*;

public class DockerEngineAPI implements RestApiSetup
{
    @Override
    public RestApi Load()
    {
        File file = new File("API Specifications\\Container\\Docker", "Docker Engine API.yaml");
        OpenAPI openAPI = new OpenAPIV3Parser().read(file.getAbsolutePath());
        OpenAPIResolver res = new OpenAPIResolver(openAPI);
        openAPI = res.resolve();
    
        RestApiConverter<OpenAPI> converter = new OpenApiToRestApi();
        RestApiSpecification restAPISpecification = converter.Convert(openAPI);
    
        RestApiConfig config = new RestApiConfig();
        String auth = ("{'username': 'joaoa13', 'password': 'Shelbycobra.427', 'email': " +
                "'joaoa13portugal@gmail.com', 'serveraddress': 'www.google.com'}").replace("'", "\"");
        config.AddParameterFilter(new ParameterFilter("X-Registry-Auth", Parameter.Location.Header).SetValue(
                Base64.encodeBase64String(auth.getBytes())).ApplyToAll()).
                AddPayloadFilter(new PayloadFilter(Path.HttpMethod.POST, "/auth").SetValue(auth).AnyMedia());
        
        ParameterFilter pFilterContainerID = new ParameterFilter("id", Parameter.Location.Path).
                SetValues("0a2b0f0ac48a", "d9ca1800dc13", "4ecbc9627c49", "da59707ddab0");
        
        config.AddParameterFilter(new ParameterFilter(pFilterContainerID.ApplyToOperation(GET, "/containers/{id}/json")));
        config.AddParameterFilter(new ParameterFilter(pFilterContainerID.ApplyToOperation(GET, "/containers/{id}/top")));
        config.AddParameterFilter(new ParameterFilter(pFilterContainerID.ApplyToOperation(GET, "/containers/{id}/changes")));
        config.AddParameterFilter(new ParameterFilter(pFilterContainerID.ApplyToOperation(POST, "/containers/{id}/resize")));
        config.AddParameterFilter(new ParameterFilter(pFilterContainerID.ApplyToOperation(POST, "/containers/{id}/start")));
        config.AddParameterFilter(new ParameterFilter(pFilterContainerID.ApplyToOperation(POST, "/containers/{id}/restart")));
        config.AddParameterFilter(new ParameterFilter(pFilterContainerID.ApplyToOperation(POST, "/containers/{id}/update")));
        config.AddParameterFilter(new ParameterFilter(pFilterContainerID.ApplyToOperation(POST, "/containers/{id}/rename")));
        config.AddParameterFilter(new ParameterFilter(pFilterContainerID.ApplyToOperation(POST, "/containers/{id}/pause")));
        config.AddParameterFilter(new ParameterFilter(pFilterContainerID.ApplyToOperation(POST, "/containers/{id}/unpause")));
        config.AddParameterFilter(new ParameterFilter(pFilterContainerID.ApplyToOperation(GET, "/containers/{id}/attach/ws")));
        config.AddParameterFilter(new ParameterFilter(pFilterContainerID.ApplyToOperation(GET, "/containers/{id}/archive")));
        config.AddParameterFilter(new ParameterFilter(pFilterContainerID.ApplyToOperation(PUT, "/containers/{id}/archive")));
        config.AddParameterFilter(new ParameterFilter(pFilterContainerID.ApplyToOperation(POST, "/containers/{id}/exec")));
        
        ParameterFilter pFilterImageName = new ParameterFilter("name", Parameter.Location.Path).
                SetValues("uniloop-api_nginx", "uniloop-api_web", "uniloop-api_postgres", "redis", "python",
                        "postgres", "tutum/nginx");
    
        config.AddParameterFilter(new ParameterFilter(pFilterImageName.ApplyToOperation(GET, "/images/{name}/json")));
        config.AddParameterFilter(new ParameterFilter(pFilterImageName.ApplyToOperation(GET, "/images/{name}/history")));
        config.AddParameterFilter(new ParameterFilter(pFilterImageName.ApplyToOperation(POST, "/images/{name}/push")));
        config.AddParameterFilter(new ParameterFilter(pFilterImageName.ApplyToOperation(POST, "/images/{name}/tag")));
    
        List<String> volumeNames = new ArrayList<>();
        volumeNames.add("7cf3d6e555379f2043e3509d72d56c0e45edbf8dea9c41537fc7de787748e814");
        volumeNames.add("627af3b5eba296ba6dcdc53904c838835c68e0a9d323fb0ba3e27d50e3c258fb");
        volumeNames.add("postgres_database");
        
        config.AddParameterFilter(new ParameterFilter("name", Parameter.Location.Path)
                .ApplyToOperation(GET, "/volumes/{name}").SetValues(volumeNames));
    
        List<String> networkIDs = new ArrayList<>();
        networkIDs.add("6e9900141643");
        networkIDs.add("99d885a57e77");
        networkIDs.add("984e49ace3a0");
        networkIDs.add("f5d70583243e");
    
        config.AddParameterFilter(new ParameterFilter("id", Parameter.Location.Path)
                .ApplyToOperation(GET, "/networks/{id}").SetValues(networkIDs));
        config.AddParameterFilter(new ParameterFilter("id", Parameter.Location.Path)
                .ApplyToOperation(POST, "/networks/{id}/connect").SetValues(networkIDs));
    
        config.AddParameterFilter(new ParameterFilter("id", Parameter.Location.Path)
                .ApplyToOperation(GET, "/nodes/{id}").SetValue("p8ytpxgpnpvostmijs7mwzxes"));
        config.AddParameterFilter(new ParameterFilter("id", Parameter.Location.Path)
                .ApplyToOperation(POST, "/nodes/{id}/update").SetValue("p8ytpxgpnpvostmijs7mwzxes"));
        
        config.AddOperationFilter(new OperationFilter(GET, "/containers/{id}/stats").Ignore());
        config.AddOperationFilter(new OperationFilter(GET, "/containers/{id}/export").Ignore());
        config.AddOperationFilter(new OperationFilter(GET, "/images/{name}/get").Ignore());
        config.AddOperationFilter(new OperationFilter(POST, "/containers/{id}/stop").Ignore());
        config.AddOperationFilter(new OperationFilter(POST, "/containers/{id}/kill").Ignore());
        config.AddOperationFilter(new OperationFilter(DELETE, "/containers/{id}").Ignore());
        config.AddOperationFilter(new OperationFilter(DELETE, "/images/{name}").Ignore());
        config.AddOperationFilter(new OperationFilter(DELETE, "/volumes/{name}").Ignore());
        config.AddOperationFilter(new OperationFilter(DELETE, "/networks/{id}").Ignore());
        config.AddOperationFilter(new OperationFilter(POST, "/networks/{id}/disconnect").Ignore());
        config.AddOperationFilter(new OperationFilter(DELETE, "/nodes/{id}").Ignore());
        config.AddOperationFilter(new OperationFilter(POST, "/swarm/leave").Ignore());
        config.AddOperationFilter(new OperationFilter(POST, "/containers/create").Ignore());
        config.AddOperationFilter(new OperationFilter(POST, "/images/create").Ignore());
        config.AddOperationFilter(new OperationFilter(POST, "/volumes/create").Ignore());
        config.AddOperationFilter(new OperationFilter(POST, "/networks/create").Ignore());
        config.AddOperationFilter(new OperationFilter(POST, "/plugins/create").Ignore());
        config.AddOperationFilter(new OperationFilter(POST, "/services/create").Ignore());
        config.AddOperationFilter(new OperationFilter(POST, "/secrets/create").Ignore());
        config.AddOperationFilter(new OperationFilter(POST, "/configs/create").Ignore());
        config.AddOperationFilter(new OperationFilter(POST, "/containers/{id}/wait").Ignore());
        config.AddOperationFilter(new OperationFilter(GET, "/containers/{id}/logs").Ignore());
        config.AddOperationFilter(new OperationFilter(GET, "/events").Ignore());
        config.AddOperationFilter(new OperationFilter(POST, "/containers/{id}/attach").Ignore());
        
        return new RestApi("Docker Engine API", restAPISpecification, config);
    }
}
