import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.parser.OpenAPIResolver;
import io.swagger.v3.parser.OpenAPIV3Parser;
import org.apache.commons.lang3.RandomUtils;
import pt.uc.dei.rest_api_robustness_tester.request.Parameter;
import pt.uc.dei.rest_api_robustness_tester.security.AuthHandlers;
import pt.uc.dei.rest_api_robustness_tester.specification.*;
import pt.uc.dei.rest_api_robustness_tester.workload.ParameterFilter;

import java.io.File;
import java.util.List;
import java.util.ArrayList;
import java.util.StringJoiner;

public class Giphy implements RestApiSetup
{
    @Override
    public RestApi Load()
    {
        //New API Key can be obtained at https://developers.giphy.com/dashboard/
        String apiKey = "8uljMCB8n7hWJD9TvsQl34TI4q7cyFgb";
    
        File file = new File("API Specifications\\Public tier 1\\Giphy", "Giphy.yaml");
        AuthHandlers authHandlers = new AuthHandlers();
        authHandlers.AddAuthHandler("api_key", requestBuilder -> requestBuilder.addParameter("api_key", apiKey));
    
        OpenAPI openAPI = new OpenAPIV3Parser().read(file.getAbsolutePath());
        OpenAPIResolver res = new OpenAPIResolver(openAPI);
        openAPI = res.resolve();
    
        RestApiConverter<OpenAPI> converter = new OpenApiToRestApi();
        RestApiSpecification restAPISpecification = converter.Convert(openAPI);
    
        RestApiConfig config = new RestApiConfig();
        config.AddParameterFilter(new ParameterFilter("rating", Parameter.Location.Query).
                SetValues("y", "g", "pg", "pg-13", "r", "unrated", "nsfw", "").ApplyToAll()).
                AddParameterFilter(new ParameterFilter("gifId", Parameter.Location.Path).
                        SetValues(GetGifIDs()).ApplyToAll()).
                AddParameterFilter(new ParameterFilter("lang", Parameter.Location.Query).
                        SetValues(GetLanguages()).ApplyToAll()).
                AddParameterFilter(new ParameterFilter("ids", Parameter.Location.Query).
                        SetValues(RandomGifIDs()).ApplyToAll()).
                AddParameterFilter(new ParameterFilter("limit", Parameter.Location.Query).
                        SetValues(GetLimits()).ApplyToAll()).
                AddParameterFilter(new ParameterFilter("offset", Parameter.Location.Query).
                        SetValues(GetOffsets()).ApplyToAll()).
                AddParameterFilter(new ParameterFilter("q", Parameter.Location.Query).
                        SetValues(GetSearchQueries()).ApplyToAll()).
                AddParameterFilter(new ParameterFilter("tag", Parameter.Location.Query).
                        SetValues(GetTags()).ApplyToAll()).
                AddParameterFilter(new ParameterFilter("s", Parameter.Location.Query).
                        SetValues(GetSearchQueries()).ApplyToAll());
    
        return new RestApi("Giphy", restAPISpecification, authHandlers, config);
    }
    
    public List<String> GetGifIDs()
    {
        List<String> list = new ArrayList<>();
        
        list.add("IcifS1qG3YFlS");
        list.add("SuJoKxDplnKve");
        list.add("3ohze3eMqHXnPIDxkY");
        list.add("EehnUUCH2JcwU");
        list.add("3ohzdTpafdPDJX1FgQ");
        list.add("l3V0x6kdXUW9M4ONq");
        list.add("Slga0v2NVP3l6");
        list.add("xUPGcL0lIfzs8MJB1m");
        list.add("xUPGcfCbPWdfAxUoHC");
        list.add("3ohzdLnRPqSAWvzYqI");
        list.add("xUPGcCKsyLEfEmbQL6");
        list.add("yzGuaUByxRtpm");
        list.add("m8AA9pDTsTS5sHHOQc");
        list.add("iIpe1w4MwTpSjniwfZ");
        list.add("dQfNc0Da2pmc1lBlyI");
        list.add("3oFzlW4yxJIGR4SrSg");
        list.add("26DOyjoSDrJn75ui4");
        list.add("ieMPmOadvWFqg");
        list.add("MTWACoGhmhSc8");
        list.add("y1GgSNQgd1Jx6");
        list.add("6qdKZFhT0VBm0");
        list.add("xUPGceFBrseLZj9irC");
        list.add("3ohzdGWXqhFcej7JPW");
        list.add("xUPGce4jjHIxMsmySk");
        list.add("xUPGcFOoKbng2T39lK");
        list.add("11XmNr1nVGksa4");
        list.add("l46Cuu0O6xJ42z66I");
        list.add("xUPGcLqQuKelrgBX9K");
        
        return list;
    }
    
    public List<String> RandomGifIDs()
    {
        List<String> list = new ArrayList<>();
    
        List<String> gifIDs = GetGifIDs();
        for(int i = 0; i < 15; i++)
        {
            StringJoiner strJoiner = new StringJoiner(",");
            for (int j = 0; j < RandomUtils.nextInt(1, 10); j++)
                strJoiner.add(gifIDs.get(RandomUtils.nextInt(0, gifIDs.size())));
            list.add(strJoiner.toString());
        }
        
        return list;
    }
    
    public List<String> GetLimits()
    {
        List<String> list = new ArrayList<>();
        
        for(int i = 0; i < 10; i++)
            list.add("" + RandomUtils.nextInt(10, 150));
        
        return list;
    }
    
    public List<String> GetOffsets()
    {
        List<String> list = new ArrayList<>();
    
        for(int i = 0; i < 5; i++)
            list.add("" + RandomUtils.nextInt(0, 10));
        
        return list;
    }
    
    public List<String> GetTags()
    {
        List<String> list = new ArrayList<>();
    
        list.add("burrito");
        list.add("hello");
        list.add("flirt");
        list.add("gay");
        list.add("weird");
        list.add("creepy");
        list.add("stupid");
        list.add("idiot");
        list.add("sexy");
        list.add("meme");
        list.add("gif");
        list.add("random");
        list.add("funny");
        list.add("wat");
        list.add("shocked");
        list.add("confused");
        list.add("nun");
        list.add("beer");
        
        return list;
    }
    
    public List<String> GetSearchQueries()
    {
        List<String> list = new ArrayList<>();
        
        list.add("balls deep");
        list.add("yellow submarine");
        list.add("blow up");
        list.add("exploding banana");
        list.add("head bum");
        list.add("spongebob squarepants");
        list.add("under the sea");
        list.add("ryan gosling");
        list.add("beer pong");
        list.add("cunt smasher");
        list.add("ass hole");
        list.add("butt poop");
        list.add("random search term");
        list.add("fancy query");
        list.add("weird cat");
        list.add("tom and jerry");
        list.add("archer bond");
        
        return list;
    }
    
    private List<String> GetLanguages()
    {
        List<String> list = new ArrayList<>();
        
        list.add("aa");
        list.add("ab");
        list.add("af");
        list.add("am");
        list.add("ar");
        list.add("as");
        list.add("ay");
        list.add("az");
        list.add("ba");
        list.add("be");
        list.add("bg");
        list.add("bh");
        list.add("bi");
        list.add("bn");
        list.add("bo");
        list.add("br");
        list.add("ca");
        list.add("co");
        list.add("cs");
        list.add("cy");
        list.add("da");
        list.add("de");
        list.add("dz");
        list.add("el");
        list.add("en");
        list.add("eo");
        list.add("es");
        list.add("et");
        list.add("eu");
        list.add("fa");
        list.add("fi");
        list.add("fj");
        list.add("fo");
        list.add("fr");
        list.add("fy");
        list.add("ga");
        list.add("gd");
        list.add("gl");
        list.add("gn");
        list.add("gu");
        list.add("ha");
        list.add("hi");
        list.add("he");
        list.add("hr");
        list.add("hu");
        list.add("hy");
        list.add("ia");
        list.add("id");
        list.add("ie");
        list.add("ik");
        list.add("in");
        list.add("is");
        list.add("it");
        list.add("iu");
        list.add("iw");
        list.add("ja");
        list.add("ji");
        list.add("jw");
        list.add("ka");
        list.add("kk");
        list.add("kl");
        list.add("km");
        list.add("kn");
        list.add("ko");
        list.add("ks");
        list.add("ku");
        list.add("ky");
        list.add("la");
        list.add("ln");
        list.add("lo");
        list.add("lt");
        list.add("lv");
        list.add("mg");
        list.add("mi");
        list.add("mk");
        list.add("ml");
        list.add("mn");
        list.add("mo");
        list.add("mr");
        list.add("ms");
        list.add("mt");
        list.add("my");
        list.add("na");
        list.add("ne");
        list.add("nl");
        list.add("no");
        list.add("oc");
        list.add("om");
        list.add("or");
        list.add("pa");
        list.add("pl");
        list.add("ps");
        list.add("pt");
        list.add("qu");
        list.add("rm");
        list.add("rn");
        list.add("ro");
        list.add("ru");
        list.add("rw");
        list.add("sa");
        list.add("sd");
        list.add("sg");
        list.add("sh");
        list.add("si");
        list.add("sk");
        list.add("sl");
        list.add("sm");
        list.add("sn");
        list.add("so");
        list.add("sq");
        list.add("sr");
        list.add("ss");
        list.add("st");
        list.add("su");
        list.add("sv");
        list.add("sw");
        list.add("ta");
        list.add("te");
        list.add("tg");
        list.add("th");
        list.add("ti");
        list.add("tk");
        list.add("tl");
        list.add("tn");
        list.add("to");
        list.add("tr");
        list.add("ts");
        list.add("tt");
        list.add("tw");
        list.add("ug");
        list.add("uk");
        list.add("ur");
        list.add("uz");
        list.add("vi");
        list.add("vo");
        list.add("wo");
        list.add("xh");
        list.add("yi");
        list.add("yo");
        list.add("za");
        list.add("zh");
        list.add("zu");
        
        return list;
    }
}
