import com.github.scribejava.core.builder.ServiceBuilder;
import com.github.scribejava.core.builder.api.DefaultApi20;
import com.github.scribejava.core.model.OAuth2AccessToken;
import com.github.scribejava.core.oauth.OAuth20Service;
import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.parser.OpenAPIResolver;
import io.swagger.v3.parser.OpenAPIV3Parser;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang3.RandomUtils;
import pt.uc.dei.rest_api_robustness_tester.Path;
import pt.uc.dei.rest_api_robustness_tester.request.AdditionalParameter;
import pt.uc.dei.rest_api_robustness_tester.request.Parameter;
import pt.uc.dei.rest_api_robustness_tester.specification.*;
import pt.uc.dei.rest_api_robustness_tester.specification.RestApiSetup;
import pt.uc.dei.rest_api_robustness_tester.utils.ElapsedTimeCounter;
import pt.uc.dei.rest_api_robustness_tester.utils.TimeUnit;
import pt.uc.dei.rest_api_robustness_tester.workload.ParameterFilter;
import pt.uc.dei.rest_api_robustness_tester.workload.RateLimiter;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

public class Calendar extends DefaultApi20 implements RestApiSetup
{
    private final OAuth20Service service;
    private final ElapsedTimeCounter elapsedTimeCounter;
    private OAuth2AccessToken accessToken = null;
    
    public Calendar()
    {
        elapsedTimeCounter = new ElapsedTimeCounter();
        service = new ServiceBuilder("984280656055-kropv57114vlt5dcjki1ooe4e7gmhqhf.apps.googleusercontent.com").
                apiSecret("5L630xQ3UkBQNri8fxaA1yKF").
                responseType("code").
                defaultScope("https://www.googleapis.com/auth/calendar " +
                        "https://www.googleapis.com/auth/calendar.events " +
                        "https://www.googleapis.com/auth/calendar.events.readonly " +
                        "https://www.googleapis.com/auth/calendar.readonly " +
                        "https://www.googleapis.com/auth/calendar.settings.readonly").
                callback("https://eden.dei.uc.pt/").
                build(this);
    }
    
    @Override
    public RestApi Load()
    {
        File file = new File("API Specifications\\Public tier 1\\Google", "Calendar.yaml");
    
        OpenAPI openAPI = new OpenAPIV3Parser().read(file.getAbsolutePath());
        OpenAPIResolver res = new OpenAPIResolver(openAPI);
        openAPI = res.resolve();
    
        RestApiConverter<OpenAPI> converter = new OpenApiToRestApi();
        RestApiSpecification restAPISpecification = converter.Convert(openAPI);
    
        RestApiConfig config = new RestApiConfig();
        config.AddAdditionalParameter(new AdditionalParameter("Authorization", Parameter.Location.Header,
                "Bearer " + GetToken())).
                SetRateLimiter(new RateLimiter(1_000_000, 1, TimeUnit.Day)).
                AddParameterFilter(new ParameterFilter("calendarId", Parameter.Location.Path).
                        SetValues(GetCallendarIDs()).ApplyToAll()).
                AddParameterFilter(new ParameterFilter("eventId", Parameter.Location.Path).
                        SetValues(GetEventIDs()).ApplyToAll()).
                AddParameterFilter(new ParameterFilter("ruleId", Parameter.Location.Path).
                        SetValues(GetRuleIDs()).ApplyToAll()).
                AddParameterFilter(new ParameterFilter("setting", Parameter.Location.Path).
                        SetValues(GetSettingIDs()).ApplyToAll()).
                AddParameterFilter(new ParameterFilter("alt", Parameter.Location.Query).Ignore()).
                AddParameterFilter(new ParameterFilter("fields", Parameter.Location.Query).Ignore()).
                AddParameterFilter(new ParameterFilter("key", Parameter.Location.Query).Ignore()).
                AddParameterFilter(new ParameterFilter("oauth_token", Parameter.Location.Query).Ignore()).
                AddParameterFilter(new ParameterFilter("prettyPrint", Parameter.Location.Query).Ignore()).
                AddParameterFilter(new ParameterFilter("quotaUser", Parameter.Location.Query).Ignore()).
                AddParameterFilter(new ParameterFilter("userIp", Parameter.Location.Query).Ignore());
        
        return new RestApi("Calendar", restAPISpecification, config);
    }
    
    @Override
    public String getAccessTokenEndpoint()
    {
        return "https://accounts.google.com/o/oauth2/token";
    }
    
    @Override
    protected String getAuthorizationBaseUrl()
    {
        return "https://accounts.google.com/o/oauth2/v2/auth";
    }
    
    private String GetToken()
    {
        if(!elapsedTimeCounter.HasStarted())
        {
            try (Scanner scan = new Scanner(System.in))
            {
                String authURL = service.getAuthorizationUrl();
                System.out.println("Open this URL in a browser: " + authURL);
                System.out.println("Enter the returned code: ");
                String code = scan.nextLine().trim();
                System.out.println(code);
                accessToken = service.getAccessToken(code);
                elapsedTimeCounter.Start();
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
        else if(elapsedTimeCounter.ElapsedSeconds() >= accessToken.getExpiresIn() - 60)
        {
            try
            {
                accessToken = service.refreshAccessToken(accessToken.getRefreshToken());
                elapsedTimeCounter.Start();
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
    
        return accessToken.getAccessToken();
    }
    
    private List<String> GetCallendarIDs()
    {
        List<String> list = new ArrayList<>();
        
        list.add("pt.uc.dei.resttester@gmail.com");
        list.add("addressbook#contacts@group.v.calendar.google.com");
        list.add("pt.portuguese#holiday@group.v.calendar.google.com");
        
        return list;
    }
    
    private List<String> GetRuleIDs()
    {
        List<String> list = new ArrayList<>();
        
        list.add("user:pt.uc.dei.resttester@gmail.com");
        list.add("default");
        
        return list;
    }
    
    private List<String> GetSettingIDs()
    {
        List<String> list = new ArrayList<>();
        
        list.add("autoAddHangouts");
        list.add("defaultEventLength");
        list.add("dateFieldOrder");
        list.add("weekStart");
        list.add("format24HourTime");
        list.add("hideInvitations");
        list.add("locale");
        list.add("remindOnRespondedEventsOnly");
        list.add("showDeclinedEvents");
        list.add("timezone");
        list.add("useKeyboardShortcuts");
        list.add("hideWeekends");
        
        return list;
    }
    
    private List<String> GetEventIDs()
    {
        List<String> list = new ArrayList<>();
        
        list.add(new String(Base64.decodeBase64("NmJxYmxtZWJwNmxiNTEwaGo2OTNtZGVndmogcHQudWMuZGVpLnJlc3R0ZXN0ZXJAbQ")).split(" ")[0]);
        list.add(new String(Base64.decodeBase64("NHBpZTU0b2R0ZjcwajZhc3MyYmgxcjAxaGsgcHQudWMuZGVpLnJlc3R0ZXN0ZXJAbQ")).split(" ")[0]);
        list.add(new String(Base64.decodeBase64("MGw5aHE0aTM5NGRpaGNxdWE5ZjA4ZzNhZm8gcHQudWMuZGVpLnJlc3R0ZXN0ZXJAbQ")).split(" ")[0]);
        list.add(new String(Base64.decodeBase64("MzRyMHA2aXRlaWtzaTgxNWRkcGt2dTBhZW8gcHQudWMuZGVpLnJlc3R0ZXN0ZXJAbQ")).split(" ")[0]);
        list.add(new String(Base64.decodeBase64("amwwa2wyZm90dDYzNHFrczUxNThvbWMzazAgcHQudWMuZGVpLnJlc3R0ZXN0ZXJAbQ")).split(" ")[0]);
        list.add(new String(Base64.decodeBase64("M3ZnN2QwNTUyOTh0YmpyMjVxZWRyYXNrbm8gcHQudWMuZGVpLnJlc3R0ZXN0ZXJAbQ")).split(" ")[0]);
        list.add(new String(Base64.decodeBase64("NjA5Y2Npc3ZhcnFqcDVlcThxamRmNjlxNjQgcHQudWMuZGVpLnJlc3R0ZXN0ZXJAbQ")).split(" ")[0]);
        list.add(new String(Base64.decodeBase64("MzlqOGxtbWhoc21sZjVpNWRtY2xsczdqaGIgcHQudWMuZGVpLnJlc3R0ZXN0ZXJAbQ")).split(" ")[0]);
        list.add(new String(Base64.decodeBase64("NW5qa3Z2MDJrMjlxZjk1OGFwZHE5MjgzOHYgcHQudWMuZGVpLnJlc3R0ZXN0ZXJAbQ")).split(" ")[0]);
        list.add(new String(Base64.decodeBase64("MzVpbzFwdWdtN3VodDZtY2hjZzRxaG9iN3AgcHQudWMuZGVpLnJlc3R0ZXN0ZXJAbQ")).split(" ")[0]);
        
        return list;
    }
}
