import com.github.scribejava.core.builder.ServiceBuilder;
import com.github.scribejava.core.builder.api.DefaultApi20;
import com.github.scribejava.core.model.OAuth2AccessToken;
import com.github.scribejava.core.oauth.OAuth20Service;
import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.parser.OpenAPIResolver;
import io.swagger.v3.parser.OpenAPIV3Parser;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang3.RandomUtils;
import pt.uc.dei.rest_api_robustness_tester.Path;
import pt.uc.dei.rest_api_robustness_tester.request.AdditionalParameter;
import pt.uc.dei.rest_api_robustness_tester.request.Parameter;
import pt.uc.dei.rest_api_robustness_tester.specification.*;
import pt.uc.dei.rest_api_robustness_tester.specification.RestApiSetup;
import pt.uc.dei.rest_api_robustness_tester.utils.ElapsedTimeCounter;
import pt.uc.dei.rest_api_robustness_tester.utils.TimeUnit;
import pt.uc.dei.rest_api_robustness_tester.workload.OperationFilter;
import pt.uc.dei.rest_api_robustness_tester.workload.ParameterFilter;
import pt.uc.dei.rest_api_robustness_tester.workload.RateLimiter;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

public class Drive extends DefaultApi20 implements RestApiSetup
{
    private final OAuth20Service service;
    private final ElapsedTimeCounter elapsedTimeCounter;
    private OAuth2AccessToken accessToken = null;
    
    public Drive()
    {
        elapsedTimeCounter = new ElapsedTimeCounter();
        service = new ServiceBuilder("984280656055-kropv57114vlt5dcjki1ooe4e7gmhqhf.apps.googleusercontent.com").
                apiSecret("5L630xQ3UkBQNri8fxaA1yKF").
                responseType("code").
                defaultScope("https://www.googleapis.com/auth/drive https://www.googleapis.com/auth/drive.appdata " +
                        "https://www.googleapis.com/auth/drive.file https://www.googleapis.com/auth/drive.metadata " +
                        "https://www.googleapis.com/auth/drive.metadata.readonly " +
                        "https://www.googleapis.com/auth/drive.photos.readonly " +
                        "https://www.googleapis.com/auth/drive.readonly https://www.googleapis.com/auth/drive.scripts").
                callback("https://eden.dei.uc.pt/").
                build(this);
    }
    
    @Override
    public RestApi Load()
    {
        File file = new File("API Specifications\\Public tier 1\\Google", "Drive.yaml");
    
        OpenAPI openAPI = new OpenAPIV3Parser().read(file.getAbsolutePath());
        OpenAPIResolver res = new OpenAPIResolver(openAPI);
        openAPI = res.resolve();
    
        RestApiConverter<OpenAPI> converter = new OpenApiToRestApi();
        RestApiSpecification restAPISpecification = converter.Convert(openAPI);
    
        RestApiConfig config = new RestApiConfig();
        config.AddAdditionalParameter(new AdditionalParameter("Authorization", Parameter.Location.Header,
                "Bearer " + GetToken())).
                SetRateLimiter(new RateLimiter(1_000, 100, TimeUnit.Second)).
                AddParameterFilter(new ParameterFilter("alt", Parameter.Location.Query).Ignore()).
                AddParameterFilter(new ParameterFilter("fields", Parameter.Location.Query).Ignore()).
                AddParameterFilter(new ParameterFilter("key", Parameter.Location.Query).Ignore()).
                AddParameterFilter(new ParameterFilter("oauth_token", Parameter.Location.Query).Ignore()).
                AddParameterFilter(new ParameterFilter("prettyPrint", Parameter.Location.Query).Ignore()).
                AddParameterFilter(new ParameterFilter("quotaUser", Parameter.Location.Query).Ignore()).
                AddParameterFilter(new ParameterFilter("userIp", Parameter.Location.Query).Ignore()).
                AddParameterFilter(new ParameterFilter("fileId", Parameter.Location.Path).
                        SetValues(GetFileIDs()).ApplyToAll()).
                AddOperationFilter(new OperationFilter(Path.HttpMethod.DELETE, "/files/{fileId}").Ignore()).
                AddOperationFilter(new OperationFilter(Path.HttpMethod.POST, "/channels/stop").Ignore()).
                AddOperationFilter(new OperationFilter(Path.HttpMethod.GET, "/about").Ignore()).
                AddOperationFilter(new OperationFilter(Path.HttpMethod.GET, "/teamdrives").Ignore()).
                AddOperationFilter(new OperationFilter(Path.HttpMethod.POST, "/teamdrives").Ignore()).
                AddOperationFilter(new OperationFilter(Path.HttpMethod.DELETE, "/teamdrives/{teamDriveId}").Ignore()).
                AddOperationFilter(new OperationFilter(Path.HttpMethod.GET, "/teamdrives/{teamDriveId}").Ignore());
        
        return new RestApi("Drive", restAPISpecification, config);
    }
    
    @Override
    public String getAccessTokenEndpoint()
    {
        return "https://accounts.google.com/o/oauth2/token";
    }
    
    @Override
    protected String getAuthorizationBaseUrl()
    {
        return "https://accounts.google.com/o/oauth2/v2/auth";
    }
    
    private String GetToken()
    {
        if(!elapsedTimeCounter.HasStarted())
        {
            try (Scanner scan = new Scanner(System.in))
            {
                String authURL = service.getAuthorizationUrl();
                System.out.println("Open this URL in a browser: " + authURL);
                System.out.println("Enter the returned code: ");
                String code = scan.nextLine().trim();
                System.out.println(code);
                accessToken = service.getAccessToken(code);
                elapsedTimeCounter.Start();
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
        else if(elapsedTimeCounter.ElapsedSeconds() >= accessToken.getExpiresIn() - 60)
        {
            try
            {
                accessToken = service.refreshAccessToken(accessToken.getRefreshToken());
                elapsedTimeCounter.Start();
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
    
        return accessToken.getAccessToken();
    }
    
    private List<String> GetFileIDs()
    {
        List<String> list = new ArrayList<>();
        
        list.add("1nOnDHd5tdeLw-lGrltv0gUrhkfB04Md0");
        list.add("1l_LC-Q3Tvu0P06dedauapYXKdk-c4aXF");
        list.add("1e5YlGh7VafzLGVA3wm-7-ctmjntObDE_");
        list.add("1DkMIyJ2Vsazc6H8HcRwbcg1e2XHrC1Xt");
        list.add("1iHA4VU4fL1wSPZmsD3TpmdnO79wuxBNx");
        list.add("1ZOtdltEz1oLKUDJjk6W9XsOpjH7sOwde");
        list.add("1KRFglMRr9UWd_A-syRDxvVSylyT6yWqZ");
        list.add("1SlqoPyRses4iza_YQBYYy5TLqkW1Z4Ak");
        list.add("1JX1CccNidD_r77_uu2jGDHaewWCUeBS5");
        list.add("1Oyy9BKQUqR84GCqqSUgbF18x3rg5Pg2d");
        list.add("1-cb0aDppRC6KZZyYW8gckxVSkjlsviAC");
        list.add("1USMqFlF7G5_bdTj5lnTHOIuF9sAxd-Oe");
        list.add("1SAmpYi20kLFoWFz4JGxPQFQmUOebuk1R");
        list.add("1V8OgpJmDPf7gjNTc6pUcVZ7ZwS_-RkYT");
        list.add("1ns6m81puu_DYzKBESPmSxjPhJt1rPAsp");
        list.add("1fGAbJZpYbAWZ5A-1vIy9UF4XXtxVjalF");
        list.add("1y19RDNWFBqtYHMwbPlhbO9EHZyxTWYZ0");
        list.add("1P0sBaSo1xFq2qVM60FGiSNwiaoM1xxsE");
        
        return list;
    }
}
