import com.github.scribejava.core.builder.ServiceBuilder;
import com.github.scribejava.core.builder.api.DefaultApi20;
import com.github.scribejava.core.model.OAuth2AccessToken;
import com.github.scribejava.core.oauth.OAuth20Service;
import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.parser.OpenAPIResolver;
import io.swagger.v3.parser.OpenAPIV3Parser;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.lang3.RandomUtils;
import pt.uc.dei.rest_api_robustness_tester.Path;
import pt.uc.dei.rest_api_robustness_tester.request.AdditionalParameter;
import pt.uc.dei.rest_api_robustness_tester.request.Parameter;
import pt.uc.dei.rest_api_robustness_tester.specification.*;
import pt.uc.dei.rest_api_robustness_tester.specification.RestApiSetup;
import pt.uc.dei.rest_api_robustness_tester.utils.ElapsedTimeCounter;
import pt.uc.dei.rest_api_robustness_tester.utils.TimeUnit;
import pt.uc.dei.rest_api_robustness_tester.workload.ParameterFilter;
import pt.uc.dei.rest_api_robustness_tester.workload.RateLimiter;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

public class YouTubeData extends DefaultApi20 implements RestApiSetup
{
    private final OAuth20Service service;
    private final ElapsedTimeCounter elapsedTimeCounter;
    private OAuth2AccessToken accessToken = null;
    
    public YouTubeData()
    {
        elapsedTimeCounter = new ElapsedTimeCounter();
        service = new ServiceBuilder("984280656055-kropv57114vlt5dcjki1ooe4e7gmhqhf.apps.googleusercontent.com").
                apiSecret("5L630xQ3UkBQNri8fxaA1yKF").
                responseType("code").
                defaultScope("https://www.googleapis.com/auth/youtube " +
                        "https://www.googleapis.com/auth/youtube.force-ssl " +
                        "https://www.googleapis.com/auth/youtube.readonly " +
                        "https://www.googleapis.com/auth/youtube.upload " +
                        "https://www.googleapis.com/auth/youtubepartner " +
                        "https://www.googleapis.com/auth/youtubepartner-channel-audit").
                callback("https://eden.dei.uc.pt/").
                build(this);
    }
    
    @Override
    public RestApi Load()
    {
        File file = new File("API Specifications\\Public tier 1\\Google", "YouTube Data.yaml");
    
        OpenAPI openAPI = new OpenAPIV3Parser().read(file.getAbsolutePath());
        OpenAPIResolver res = new OpenAPIResolver(openAPI);
        openAPI = res.resolve();
    
        RestApiConverter<OpenAPI> converter = new OpenApiToRestApi();
        RestApiSpecification restAPISpecification = converter.Convert(openAPI);
    
        RestApiConfig config = new RestApiConfig();
        config.AddAdditionalParameter(new AdditionalParameter("Authorization", Parameter.Location.Header,
                "Bearer " + GetToken())).
                SetRateLimiter(new RateLimiter(1_000_000, 1, TimeUnit.Day)).
                AddParameterFilter(new ParameterFilter("alt", Parameter.Location.Query).Ignore()).
                AddParameterFilter(new ParameterFilter("fields", Parameter.Location.Query).Ignore()).
                AddParameterFilter(new ParameterFilter("key", Parameter.Location.Query).Ignore()).
                AddParameterFilter(new ParameterFilter("oauth_token", Parameter.Location.Query).Ignore()).
                AddParameterFilter(new ParameterFilter("prettyPrint", Parameter.Location.Query).Ignore()).
                AddParameterFilter(new ParameterFilter("quotaUser", Parameter.Location.Query).Ignore()).
                AddParameterFilter(new ParameterFilter("userIp", Parameter.Location.Query).Ignore());
        
        return new RestApi("YouTubeData", restAPISpecification, config);
    }
    
    @Override
    public String getAccessTokenEndpoint()
    {
        return "https://accounts.google.com/o/oauth2/token";
    }
    
    @Override
    protected String getAuthorizationBaseUrl()
    {
        return "https://accounts.google.com/o/oauth2/v2/auth";
    }
    
    private String GetToken()
    {
        if(!elapsedTimeCounter.HasStarted())
        {
            try (Scanner scan = new Scanner(System.in))
            {
                String authURL = service.getAuthorizationUrl();
                System.out.println("Open this URL in a browser: " + authURL);
                System.out.println("Enter the returned code: ");
                String code = scan.nextLine().trim();
                System.out.println(code);
                accessToken = service.getAccessToken(code);
                elapsedTimeCounter.Start();
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
        else if(elapsedTimeCounter.ElapsedSeconds() >= accessToken.getExpiresIn() - 60)
        {
            try
            {
                accessToken = service.refreshAccessToken(accessToken.getRefreshToken());
                elapsedTimeCounter.Start();
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
    
        return accessToken.getAccessToken();
    }
    
    private List<String> GetCallendarIDs()
    {
        List<String> list = new ArrayList<>();
        
        list.add("");
        
        return list;
    }
}
