import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.parser.OpenAPIResolver;
import io.swagger.v3.parser.OpenAPIV3Parser;
import pt.uc.dei.rest_api_robustness_tester.Path;
import pt.uc.dei.rest_api_robustness_tester.request.Parameter;
import pt.uc.dei.rest_api_robustness_tester.security.AuthHandlers;
import pt.uc.dei.rest_api_robustness_tester.specification.*;
import pt.uc.dei.rest_api_robustness_tester.specification.RestApiSetup;
import pt.uc.dei.rest_api_robustness_tester.workload.OperationFilter;
import pt.uc.dei.rest_api_robustness_tester.workload.ParameterFilter;

import java.io.File;

public class Slack implements RestApiSetup
{
    @Override
    public RestApi Load()
    {
        //Slack workspace name: acme-ugu7279.slack.com
        //Manage Slack API at https://api.slack.com/apps/AQH8C3TM2
    
        //OAuth handling explained at https://github.com/dmfs/oauth2-essentials
    
        File file = new File("API Specifications\\Public tier 1\\Slack", "Slack.yaml");
        AuthHandlers authHandlers = new AuthHandlers();
        authHandlers.AddAuthHandler("slackAuth", requestBuilder ->
                {
//                    HttpRequestExecutor executor = new HttpUrlConnectionExecutor();
//
//                    OAuth2AuthorizationProvider provider = new BasicOAuth2AuthorizationProvider(
//                    URI.create("https://slack.com/oauth/authorize"),
//                    URI.create("https://slack.com/api/oauth.access"),
//                    new Duration(1,0,3600));
//
//                    OAuth2ClientCredentials credentials = new BasicOAuth2ClientCredentials(
//                    "833272068324.833284129716", "93bc16d26622d0ff41fac6af96bcd255");
//
//                    Uri redirectURL = new OpaqueUri(new StringScheme("HTTP"), new EncodedPath(new Precoded("localhost")));
//                    OAuth2Client client = new BasicOAuth2Client(provider, credentials, redirectURL);
//
//                    try
//                    {
//                        OAuth2InteractiveGrant grant = new AuthorizationCodeGrant(client, new BasicScope("scope"));
//
//                        OAuth2AccessToken token = grant.withRedirect(redirectURL).accessToken(executor);
//
//                        requestBuilder.addHeader("Authorization", "Bearer " + token.accessToken());
//
//                    }
//                    catch (Exception e)
//                    {
//                        e.printStackTrace();
//                    }
                    requestBuilder.addHeader("Authorization", "Bearer " +
                            "xoxp-833272068324-833743563541-848806939985-2b4bccc895403f0ee3eebef32ad87da1");
                });
    
        OpenAPI openAPI = new OpenAPIV3Parser().read(file.getAbsolutePath());
        OpenAPIResolver res = new OpenAPIResolver(openAPI);
        openAPI = res.resolve();
    
        RestApiConverter<OpenAPI> converter = new OpenApiToRestApi();
        RestApiSpecification restAPISpecification = converter.Convert(openAPI);
    
        RestApiConfig config = new RestApiConfig();
        config.AddParameterFilter(new ParameterFilter("token", Parameter.Location.Header).Ignore().ApplyToAll()).
                AddParameterFilter(new ParameterFilter("token", Parameter.Location.Query).Ignore().ApplyToAll()).
                AddOperationFilter(new OperationFilter(Path.HttpMethod.GET, "/auth.revoke").Ignore()).
                AddOperationFilter(new OperationFilter(Path.HttpMethod.GET, "/oauth.access").Ignore()).
                AddOperationFilter(new OperationFilter(Path.HttpMethod.GET, "/oauth.token").Ignore());
        
        return new RestApi("Slack", restAPISpecification, authHandlers, config);
    }
}
