import com.github.scribejava.core.builder.ServiceBuilder;
import com.github.scribejava.core.builder.api.DefaultApi20;
import com.github.scribejava.core.model.OAuth2AccessToken;
import com.github.scribejava.core.oauth.OAuth20Service;
import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.parser.OpenAPIResolver;
import io.swagger.v3.parser.OpenAPIV3Parser;
import org.apache.commons.lang3.RandomUtils;
import pt.uc.dei.rest_api_robustness_tester.Path;
import pt.uc.dei.rest_api_robustness_tester.request.AdditionalParameter;
import pt.uc.dei.rest_api_robustness_tester.request.Parameter;
import pt.uc.dei.rest_api_robustness_tester.specification.*;
import pt.uc.dei.rest_api_robustness_tester.specification.RestApiSetup;
import pt.uc.dei.rest_api_robustness_tester.utils.ElapsedTimeCounter;
import pt.uc.dei.rest_api_robustness_tester.utils.TimeUnit;
import pt.uc.dei.rest_api_robustness_tester.workload.ParameterFilter;
import pt.uc.dei.rest_api_robustness_tester.workload.RateLimiter;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import java.util.Scanner;

public class Spotify extends DefaultApi20 implements RestApiSetup
{
    private final OAuth20Service service;
    private final ElapsedTimeCounter elapsedTimeCounter;
    private OAuth2AccessToken accessToken = null;
    
    public Spotify()
    {
        elapsedTimeCounter = new ElapsedTimeCounter();
        service = new ServiceBuilder("ab535b28d8d843358cfd1086357a721a").
                apiSecret("aeebe552bd0c4bbd84cd8e8181f3cf12").
                responseType("code").
                defaultScope("ugc-image-upload user-modify-playback-state user-read-playback-state " +
                        "user-read-currently-playing user-top-read user-read-recently-played user-library-modify " +
                        "user-library-read user-follow-modify user-follow-read playlist-read-private " +
                        "playlist-modify-public playlist-modify-private playlist-read-collaborative user-read-private " +
                        "user-read-email app-remote-control streaming").
                callback("https://eden.dei.uc.pt/").
                build(this);
    }
    
    @Override
    public RestApi Load()
    {
        File file = new File("API Specifications\\Public tier 1\\Spotify", "Spotify.yaml");
    
        OpenAPI openAPI = new OpenAPIV3Parser().read(file.getAbsolutePath());
        OpenAPIResolver res = new OpenAPIResolver(openAPI);
        openAPI = res.resolve();
    
        RestApiConverter<OpenAPI> converter = new OpenApiToRestApi();
        RestApiSpecification restAPISpecification = converter.Convert(openAPI);
    
        RestApiConfig config = new RestApiConfig();
        config.AddAdditionalParameter(new AdditionalParameter("Authorization", Parameter.Location.Header,
                "Bearer " + GetToken())).
                SetRateLimiter(new RateLimiter(100, 20, TimeUnit.Second)).
                AddParameterFilter(new ParameterFilter("category_id", Parameter.Location.Path).
                        SetValues(GetCategoryIDs()).ApplyToAll()).
                AddParameterFilter(new ParameterFilter("user_id", Parameter.Location.Path).
                        SetValues(GetUserIDs()).ApplyToAll()).
                AddParameterFilter(new ParameterFilter("playlist_id", Parameter.Location.Path).
                        SetValues(GetPlaylistIDs()).ApplyToAll()).
                AddParameterFilter(new ParameterFilter("id", Parameter.Location.Path).
                        SetValues(GetTrackIDs()).ApplyToOperation(Path.HttpMethod.GET, "/tracks/{id}")).
                AddParameterFilter(new ParameterFilter("id", Parameter.Location.Path).
                        SetValues(GetAlbumIDs()).ApplyToOperation(Path.HttpMethod.GET, "/albums/{id}")).
                AddParameterFilter(new ParameterFilter("id", Parameter.Location.Path).
                        SetValues(GetAlbumIDs()).ApplyToOperation(Path.HttpMethod.GET, "/albums/{id}/tracks")).
                AddParameterFilter(new ParameterFilter("id", Parameter.Location.Path).
                        SetValues(GetArtistIDs()).ApplyToOperation(Path.HttpMethod.GET, "/artists/{id}")).
                AddParameterFilter(new ParameterFilter("id", Parameter.Location.Path).
                        SetValues(GetArtistIDs()).ApplyToOperation(Path.HttpMethod.GET, "/artists/{id}/albums")).
                AddParameterFilter(new ParameterFilter("id", Parameter.Location.Path).
                        SetValues(GetArtistIDs()).ApplyToOperation(Path.HttpMethod.GET, "/artists/{id}/related-artists")).
                AddParameterFilter(new ParameterFilter("id", Parameter.Location.Path).
                        SetValues(GetArtistIDs()).ApplyToOperation(Path.HttpMethod.GET, "/artists/{id}/top-tracks")).
                AddParameterFilter(new ParameterFilter("market", Parameter.Location.Query).
                        SetValues(GetCountries()).ApplyToAll()).
                AddParameterFilter(new ParameterFilter("country", Parameter.Location.Query).
                        SetValues(GetCountries()).ApplyToAll()).
                AddParameterFilter(new ParameterFilter("locale", Parameter.Location.Query).
                        SetValues(JoinCountriesAndLanguages()).ApplyToAll()).
                AddParameterFilter(new ParameterFilter("Accept", Parameter.Location.Header).Ignore()).
                AddParameterFilter(new ParameterFilter("album_type", Parameter.Location.Query).
                        SetValues("album", "single", "compilation").ApplyToAll());
        
        return new RestApi("Spotify", restAPISpecification, config);
    }
    
    @Override
    public String getAccessTokenEndpoint()
    {
        return "https://accounts.spotify.com/api/token";
    }
    
    @Override
    protected String getAuthorizationBaseUrl()
    {
        return "https://accounts.spotify.com/authorize";
    }
    
    private String GetToken()
    {
        if(!elapsedTimeCounter.HasStarted())
        {
            try (Scanner scan = new Scanner(System.in))
            {
                String authURL = service.getAuthorizationUrl();
                System.out.println("Open this URL in a browser: " + authURL);
                System.out.println("Enter the returned code: ");
                String code = scan.nextLine().trim();
                System.out.println(code);
                accessToken = service.getAccessToken(code);
                elapsedTimeCounter.Start();
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
        else if(elapsedTimeCounter.ElapsedSeconds() >= accessToken.getExpiresIn() - 60)
        {
            try
            {
                accessToken = service.refreshAccessToken(accessToken.getRefreshToken());
                elapsedTimeCounter.Start();
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
    
        return accessToken.getAccessToken();
    }
    
    private List<String> GetUserIDs()
    {
        List<String> list = new ArrayList<>();
        
        list.add("12174004901");
        list.add("1170548391");
        list.add("chancleteando");
        list.add("22qppzmfta7kblner6fiu3mby");
        list.add("irinathediva");
        list.add("joanamlimao");
        list.add("joyrydr");
        list.add("motörhead");
        list.add("tgcl7r2tqhx5nha4ob0u6dp1q");
        list.add("telmorc");
        list.add("1171811118");
        list.add("12166594516");
        list.add("1130600560");
        list.add("1278747774");
        list.add("xingenious");
        list.add("kbk97");
        list.add("1182301114");
        list.add("vikiluna");
        
        return list;
    }
    
    private List<String> GetPlaylistIDs()
    {
        List<String> list = new ArrayList<>();
        
        list.add("41RoABmM8MBVzwE7KmNWWT");
        list.add("12AWx75iz8Zz0IHpQHBygn");
        list.add("1MCkjbfzSaGJjk7RwTt743");
        list.add("4YDxqzLehycylYnJIiteEH");
        list.add("5ufjzqyoN6lhLLKh4erZ6m");
        list.add("3GxEtVFm2DUatsMQiUxL0z");
        list.add("37i9dQZF1DWXmlLSKkfdAk");
        list.add("3Bbp1OwIl9nOsYpGtGg7GT");
        list.add("7vwWoCkHp5qxITaax1IB4Q");
        list.add("7kMSdczVCUtAxJdImb08bL");
        list.add("2Th46RL9u1saQB05ZUkCms");
        list.add("1xmCvoZMh6y6kULtemceFl");
        list.add("37i9dQZF1DWZUozJiHy44Y");
        list.add("2rxJ32eb6JY25h8iSvpfGM");
        list.add("3WeeqBHulCFnr1oUrfdTLN");
        list.add("2lon3ep7gGld5ggjwTL030");
        list.add("69hUNF28p283hO7P1zeZvX");
        list.add("7wrA5z6Bd0uBWiIA7C8014");
        list.add("17Ic2ME8zoTR1gibBkt0nI");
        list.add("6r4fHmeNqRmyZNoFABXa79");
        list.add("3yeRLQUVT2AqfEdDq9GdWK");
        list.add("1zrZa3KoxpYjPzTv72Jby8");
        list.add("1X2bBjyzNMmlndipuIldMn");
        list.add("6VX92aPktKlOjqKYzdxSQI");
        list.add("7xRVn4q3S4bsOc4G5x36Ci");
        list.add("3keu53pezZUCdc0Zf9dFEl");
        list.add("11DPqDnKmvYfyGidObMbTL");
        list.add("64mvVM8fjIOcoVphlfqrSQ");
        list.add("0BH9juLlpytTKGNcSs3qEn");
        list.add("3Zr5fFWwHTFioucSzaF05q");
        list.add("6Yz6Ze87xoWSC3HdyqXQ9s");
        list.add("6nMth5Hx2Qn45B8pt1jTUg");
        list.add("1zSrR8U7m23QfmDvcYZGog");
        list.add("49BzZ2cIR3wp7syG9A031G");
        list.add("2PdG1ivZte7XEBxbueanBw");
        list.add("37i9dQZF1DXdnGF35OawbN");
        list.add("79xlKoIO91SrBeBRxHlUWj");
        list.add("41gkxKVDh2lG4z0TyWQO5U");
        list.add("30SszyMCHpqGxKCl96hKiW");
        list.add("7kH4RsWefO0FW1AHBXMQhz");
        list.add("77yeIF9idgAtji5EUzcmbF");
        list.add("1Czas2wF1TUDzk2sL7l290");
        list.add("3y16zDfUHelxBj9qsNf3hr");
        list.add("46dbsr1DmIQTf5f3DQXKEY");
        list.add("5MnrqFfJZ0IPfQ4TXBBvTR");
        list.add("2DGICPTEXCKRS374vZZsm9");
        list.add("3EpInQmB1oe1wMceMVPp8k");
        list.add("7e2ssAELWMYAIEzZIMnkfC");
        list.add("7LG5ejPPul2BZVWdkpqVvQ");
        list.add("4ozvRrHgk23R4syZv52XNz");
        list.add("1i3zXPKIBb1vaSkoGeKMkd");
        list.add("3aqjwVRVq3To5VVkdONBI3");
        list.add("0l9vxryBTxxqZ6rsXfktpN");
        list.add("37i9dQZF1DX70RN3TfWWJh");
        list.add("7q9IHlB5WwNM8o4k3tneb0");
        list.add("4m6VZP91Q9sAc3jfe9JnwK");
        list.add("0Su4ZQssLWg2MEn4lrcE7Z");
        list.add("59wqFOVsOKgSMog76qIo95");
        list.add("0pxJgHTNwk1H8CozQ6rKBO");
        list.add("3HSzH9qM1zjQ3erlIUDrLc");
        list.add("0XifCzlnO88qhnsDRkkpXa");
        list.add("3qMc7zAHC2rzeR1DjGDdPw");
        list.add("1fBUv334UEidKOsb9a43Ml");
        list.add("2whYAnVptqG0Ta9eMUjJ2J");
        
        return list;
    }
    
    private List<String> GetArtistIDs()
    {
        List<String> list = new ArrayList<>();
        
        list.add("0k17h0D3J5VfsdmQ1iZtE9");
        list.add("1dfeR4HaWDbWqFHLkxsg1d");
        list.add("432R46LaYsJZV2Gmc4jUV5");
        list.add("0yNLKJebCb8Aueb54LYya3");
        list.add("3iTsJGG39nMg9YiolUgLMQ");
        list.add("7Fo8TAyGJr4VmhE68QamMf");
        list.add("7cKtqv9cYVlOwnuCFH95ce");
        list.add("1aX2dmV8XoHYCOQRxjPESG");
        list.add("28yIUgPYZqgJoNZjeBvrHn");
        list.add("2oyWkw7sq99yqj12hVUHtw");
        list.add("1mZu3rO7qSD09GdDpePHhY");
        list.add("6hN9F0iuULZYWXppob22Aj");
        list.add("6loBF9iQdE11WSX29fNKqY");
        list.add("0WUphJOGHE5i95IeR87hsO");
        list.add("0L9xkvBPcEp1nrhDrodxc5");
        list.add("2wpWOzQE5TpA0dVnh5YD08");
        list.add("3iUjRVvYCsMfz7tuAQtBDI");
        list.add("0lZoBs4Pzo7R89JM9lxwoT");
        
        return list;
    }
    
    private List<String> GetAlbumIDs()
    {
        List<String> list = new ArrayList<>();
        
        list.add("4l4ry48IeNbMCZl5PoI6Lo");
        list.add("7wzStEU2keGohEu8jpVMZW");
        list.add("6uvBKDGlJAYLH5Vy6RQVsc");
        list.add("70pjSyIDOZ5G5blR6aNXkQ");
        list.add("391ScNR3xKywWSpfDwP3n0");
        list.add("0cbpcdI4UySacPh5RCpDfo");
        list.add("7Hm47KQv4uPp1ijUhmyqea");
        list.add("1beCGiP2LRkyDq6zmbyF6w");
        list.add("4YXo7p7aubyVIbNLoVlBp9");
        list.add("3TWyZakhbJnds6Qx2uXIiC");
        list.add("0FU4Eo42Oyg1We3eRrOf4m");
        list.add("0GJLjFeSsctGR8YeDnchOr");
        list.add("5LpKLoUmnnE3TVzoEjdrmm");
        list.add("2vO03vgGm5VWdTk6D82ykb");
        list.add("2FLZD1SiL0Lp0xZHsTBDC4");
        list.add("0gRbN3KQdhFvRRmrsZAPs8");
        list.add("1yz576NWGtvvV9Sz1xDTF1");
        list.add("6Or545oGksoE03XAHZNHKv");
        
        return list;
    }
    
    private List<String> GetTrackIDs()
    {
        List<String> list = new ArrayList<>();
        
        list.add("5MtnbY0fACKCYyMAsSPmSf");
        list.add("1T5fan6ZHgj8izJaHOkXlX");
        list.add("0rp0LgCK6BnYbEn1xj1qzH");
        list.add("7Fi3aMIXWBnzGD1LuFThML");
        list.add("24yHHIYVMYwN4oqfpiIh6I");
        list.add("4k2CdsRopiwCKqO6rOSiVY");
        list.add("06HjuPU7sr8NF2jSqqq5DG");
        list.add("2RyguUkAyXZCy9fPm9t287");
        list.add("2y6krW4iylDWiKVxTzUJcE");
        list.add("4XE9Ggjr33rRGbjDHbGaTQ");
        list.add("1GdkGsln9i2RYro5gTAht7");
        list.add("4jAIqgrPjKLTY9Gbez25Qb");
        list.add("5IhzJOXNE7ki0IIJbZbnGq");
        list.add("72kIC2q0kaOleQ3uCWU6eN");
        list.add("72mDETn3Ba8yVewAcqnlqQ");
        list.add("6hv1AUktgRGpY6coub12NG");
        list.add("6ageUeraq8yY687IBlpeEF");
        list.add("5MIyzgrAUZ5TryLLF30hcY");
        
        return list;
    }
    
    private List<String> GetCategoryIDs()
    {
        List<String> list = new ArrayList<>();
        
        list.add("toplists");
        list.add("pop");
        list.add("hiphop");
        list.add("chill");
        list.add("mood");
        list.add("brazilian");
        list.add("latin");
        list.add("edm_dance");
        list.add("rock");
        list.add("indie_alt");
        list.add("wellness");
        list.add("party");
        list.add("thirdparty");
        list.add("workout");
        list.add("focus");
        list.add("decades");
        list.add("funk");
        list.add("reggae");
        list.add("rnb");
        list.add("jazz");
        
        return list;
    }
    
    private List<String> GetCountries()
    {
        List<String> list = new ArrayList<>();
    
        list.add("AF");
        list.add("AX");
        list.add("AL");
        list.add("DZ");
        list.add("AS");
        list.add("AD");
        list.add("AO");
        list.add("AI");
        list.add("AQ");
        list.add("AG");
        list.add("AR");
        list.add("AM");
        list.add("AW");
        list.add("AU");
        list.add("AT");
        list.add("AZ");
        list.add("BH");
        list.add("BS");
        list.add("BD");
        list.add("BB");
        list.add("BY");
        list.add("BE");
        list.add("BZ");
        list.add("BJ");
        list.add("BM");
        list.add("BT");
        list.add("BO");
        list.add("BQ");
        list.add("BA");
        list.add("BW");
        list.add("BV");
        list.add("BR");
        list.add("IO");
        list.add("BN");
        list.add("BG");
        list.add("BF");
        list.add("BI");
        list.add("KH");
        list.add("CM");
        list.add("CA");
        list.add("CV");
        list.add("KY");
        list.add("CF");
        list.add("TD");
        list.add("CL");
        list.add("CN");
        list.add("CX");
        list.add("CC");
        list.add("CO");
        list.add("KM");
        list.add("CG");
        list.add("CD");
        list.add("CK");
        list.add("CR");
        list.add("CI");
        list.add("HR");
        list.add("CU");
        list.add("CW");
        list.add("CY");
        list.add("CZ");
        list.add("DK");
        list.add("DJ");
        list.add("DM");
        list.add("DO");
        list.add("EC");
        list.add("EG");
        list.add("SV");
        list.add("GQ");
        list.add("ER");
        list.add("EE");
        list.add("ET");
        list.add("FK");
        list.add("FO");
        list.add("FJ");
        list.add("FI");
        list.add("FR");
        list.add("GF");
        list.add("PF");
        list.add("TF");
        list.add("GA");
        list.add("GM");
        list.add("GE");
        list.add("DE");
        list.add("GH");
        list.add("GI");
        list.add("GR");
        list.add("GL");
        list.add("GD");
        list.add("GP");
        list.add("GU");
        list.add("GT");
        list.add("GG");
        list.add("GN");
        list.add("GW");
        list.add("GY");
        list.add("HT");
        list.add("HM");
        list.add("VA");
        list.add("HN");
        list.add("HK");
        list.add("HU");
        list.add("IS");
        list.add("IN");
        list.add("ID");
        list.add("IR");
        list.add("IQ");
        list.add("IE");
        list.add("IM");
        list.add("IL");
        list.add("IT");
        list.add("JM");
        list.add("JP");
        list.add("JE");
        list.add("JO");
        list.add("KZ");
        list.add("KE");
        list.add("KI");
        list.add("KP");
        list.add("KR");
        list.add("KW");
        list.add("KG");
        list.add("LA");
        list.add("LV");
        list.add("LB");
        list.add("LS");
        list.add("LR");
        list.add("LY");
        list.add("LI");
        list.add("LT");
        list.add("LU");
        list.add("MO");
        list.add("MK");
        list.add("MG");
        list.add("MW");
        list.add("MY");
        list.add("MV");
        list.add("ML");
        list.add("MT");
        list.add("MH");
        list.add("MQ");
        list.add("MR");
        list.add("MU");
        list.add("YT");
        list.add("MX");
        list.add("FM");
        list.add("MD");
        list.add("MC");
        list.add("MN");
        list.add("ME");
        list.add("MS");
        list.add("MA");
        list.add("MZ");
        list.add("MM");
        list.add("NA");
        list.add("NR");
        list.add("NP");
        list.add("NL");
        list.add("NC");
        list.add("NZ");
        list.add("NI");
        list.add("NE");
        list.add("NG");
        list.add("NU");
        list.add("NF");
        list.add("MP");
        list.add("NO");
        list.add("OM");
        list.add("PK");
        list.add("PW");
        list.add("PS");
        list.add("PA");
        list.add("PG");
        list.add("PY");
        list.add("PE");
        list.add("PH");
        list.add("PN");
        list.add("PL");
        list.add("PT");
        list.add("PR");
        list.add("QA");
        list.add("RE");
        list.add("RO");
        list.add("RU");
        list.add("RW");
        list.add("BL");
        list.add("SH");
        list.add("KN");
        list.add("LC");
        list.add("MF");
        list.add("PM");
        list.add("VC");
        list.add("WS");
        list.add("SM");
        list.add("ST");
        list.add("SA");
        list.add("SN");
        list.add("RS");
        list.add("SC");
        list.add("SL");
        list.add("SG");
        list.add("SX");
        list.add("SK");
        list.add("SI");
        list.add("SB");
        list.add("SO");
        list.add("ZA");
        list.add("GS");
        list.add("SS");
        list.add("ES");
        list.add("LK");
        list.add("SD");
        list.add("SR");
        list.add("SJ");
        list.add("SZ");
        list.add("SE");
        list.add("CH");
        list.add("SY");
        list.add("TW");
        list.add("TJ");
        list.add("TZ");
        list.add("TH");
        list.add("TL");
        list.add("TG");
        list.add("TK");
        list.add("TO");
        list.add("TT");
        list.add("TN");
        list.add("TR");
        list.add("TM");
        list.add("TC");
        list.add("TV");
        list.add("UG");
        list.add("UA");
        list.add("AE");
        list.add("GB");
        list.add("US");
        list.add("UM");
        list.add("UY");
        list.add("UZ");
        list.add("VU");
        list.add("VE");
        list.add("VN");
        list.add("VG");
        list.add("VI");
        list.add("WF");
        list.add("EH");
        list.add("YE");
        list.add("ZM");
        list.add("ZW");
        
        return list;
    }
    
    private List<String> GetLanguages()
    {
        List<String> list = new ArrayList<>();
    
        list.add("aa");
        list.add("ab");
        list.add("af");
        list.add("am");
        list.add("ar");
        list.add("as");
        list.add("ay");
        list.add("az");
        list.add("ba");
        list.add("be");
        list.add("bg");
        list.add("bh");
        list.add("bi");
        list.add("bn");
        list.add("bo");
        list.add("br");
        list.add("ca");
        list.add("co");
        list.add("cs");
        list.add("cy");
        list.add("da");
        list.add("de");
        list.add("dz");
        list.add("el");
        list.add("en");
        list.add("eo");
        list.add("es");
        list.add("et");
        list.add("eu");
        list.add("fa");
        list.add("fi");
        list.add("fj");
        list.add("fo");
        list.add("fr");
        list.add("fy");
        list.add("ga");
        list.add("gd");
        list.add("gl");
        list.add("gn");
        list.add("gu");
        list.add("ha");
        list.add("hi");
        list.add("he");
        list.add("hr");
        list.add("hu");
        list.add("hy");
        list.add("ia");
        list.add("id");
        list.add("ie");
        list.add("ik");
        list.add("in");
        list.add("is");
        list.add("it");
        list.add("iu");
        list.add("iw");
        list.add("ja");
        list.add("ji");
        list.add("jw");
        list.add("ka");
        list.add("kk");
        list.add("kl");
        list.add("km");
        list.add("kn");
        list.add("ko");
        list.add("ks");
        list.add("ku");
        list.add("ky");
        list.add("la");
        list.add("ln");
        list.add("lo");
        list.add("lt");
        list.add("lv");
        list.add("mg");
        list.add("mi");
        list.add("mk");
        list.add("ml");
        list.add("mn");
        list.add("mo");
        list.add("mr");
        list.add("ms");
        list.add("mt");
        list.add("my");
        list.add("na");
        list.add("ne");
        list.add("nl");
        list.add("no");
        list.add("oc");
        list.add("om");
        list.add("or");
        list.add("pa");
        list.add("pl");
        list.add("ps");
        list.add("pt");
        list.add("qu");
        list.add("rm");
        list.add("rn");
        list.add("ro");
        list.add("ru");
        list.add("rw");
        list.add("sa");
        list.add("sd");
        list.add("sg");
        list.add("sh");
        list.add("si");
        list.add("sk");
        list.add("sl");
        list.add("sm");
        list.add("sn");
        list.add("so");
        list.add("sq");
        list.add("sr");
        list.add("ss");
        list.add("st");
        list.add("su");
        list.add("sv");
        list.add("sw");
        list.add("ta");
        list.add("te");
        list.add("tg");
        list.add("th");
        list.add("ti");
        list.add("tk");
        list.add("tl");
        list.add("tn");
        list.add("to");
        list.add("tr");
        list.add("ts");
        list.add("tt");
        list.add("tw");
        list.add("ug");
        list.add("uk");
        list.add("ur");
        list.add("uz");
        list.add("vi");
        list.add("vo");
        list.add("wo");
        list.add("xh");
        list.add("yi");
        list.add("yo");
        list.add("za");
        list.add("zh");
        list.add("zu");
        
        return list;
    }
    
    private List<String> JoinCountriesAndLanguages()
    {
        List<String> list = new ArrayList<>();
        
        List<String> countries = GetCountries();
        
        for(String language : GetLanguages())
            list.add(language + "_" + countries.get(RandomUtils.nextInt(0, countries.size())));
        
        return list;
    }
}
