import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.parser.OpenAPIResolver;
import io.swagger.v3.parser.OpenAPIV3Parser;
import pt.uc.dei.rest_api_robustness_tester.specification.OpenApiToRestApi;
import pt.uc.dei.rest_api_robustness_tester.specification.RestApi;
import pt.uc.dei.rest_api_robustness_tester.specification.RestApiConverter;
import pt.uc.dei.rest_api_robustness_tester.specification.RestApiSpecification;
import pt.uc.dei.rest_api_robustness_tester.specification.RestApiSetup;
import pt.uc.dei.rest_api_robustness_tester.specification.RestApiConfig;
import pt.uc.dei.rest_api_robustness_tester.workload.ParameterFilter;
import pt.uc.dei.rest_api_robustness_tester.request.AdditionalParameter;
import pt.uc.dei.rest_api_robustness_tester.request.Parameter;

import java.io.File;
import java.util.*;

public class Trello implements RestApiSetup
{
    @Override
    public RestApi Load()
    {
        //New API Key can be obtained at https://trello.com/app-key
        String key = "5cedace1dd5cb73472f11f05c374fcd0";
		String token = "42908cee6f5fcb3b20f39d64e9c7ff791a80a556b84020a9f51bcbb048f2e8b5";
    
        File file = new File("API Specifications\\Public tier 1\\Trello", "Trello.yaml");
    
        OpenAPI openAPI = new OpenAPIV3Parser().read(file.getAbsolutePath());
        OpenAPIResolver res = new OpenAPIResolver(openAPI);
        openAPI = res.resolve();
    
        RestApiConverter<OpenAPI> converter = new OpenApiToRestApi();
        RestApiSpecification restAPISpecification = converter.Convert(openAPI);
    
        RestApiConfig config = new RestApiConfig();
        config.AddParameterFilter(new ParameterFilter("key", Parameter.Location.Query).Ignore().ApplyToAll()).
                AddParameterFilter(new ParameterFilter("token", Parameter.Location.Query).Ignore().ApplyToAll()).
                AddAdditionalParameter(new AdditionalParameter("key", Parameter.Location.Query, key).ApplyToAll()).
                AddAdditionalParameter(new AdditionalParameter("token", Parameter.Location.Query, token).ApplyToAll()).
				AddParameterFilter(new ParameterFilter("idBoard", Parameter.Location.Path).SetValue("V6wZgvHv").ApplyToAll()).
				AddParameterFilter(new ParameterFilter("idAction", Parameter.Location.Path).SetValues(GetActionTypes()).ApplyToAll()).
				AddParameterFilter(new ParameterFilter("idCard", Parameter.Location.Path).SetValues(GetCardIds()).ApplyToAll()).
				AddParameterFilter(new ParameterFilter("idMember", Parameter.Location.Path).SetValue("firstlast143").ApplyToAll());
    
        return new RestApi("Trello", restAPISpecification, config);
    }
	
	private List<String> GetActionTypes()
	{
		List<String> idAction = new ArrayList<>();
		idAction.add("acceptEnterpriseJoinRequest");
		idAction.add("addAttachmentToCard");
		idAction.add("addChecklistToCard");
		idAction.add("addLabelToCard");
		idAction.add("addMemberToBoard");
		idAction.add("addMemberToCard");
		idAction.add("addMemberToOrganization");
		idAction.add("addOrganizationToEnterprise");
		idAction.add("addToEnterprisePluginWhitelist");
		idAction.add("addToOrganizationBoard");
		idAction.add("commentCard");
		idAction.add("convertToCardFromCheckItem");
		idAction.add("copyBoard");
		idAction.add("copyCard");
		idAction.add("copyChecklist");
		idAction.add("createLabel");
		idAction.add("copyCommentCard");
		idAction.add("createBoard");
		idAction.add("createBoardInvitation");
		idAction.add("createBoardPreference");
		idAction.add("createCard");
		idAction.add("createList");
		idAction.add("createOrganization");
		idAction.add("createOrganizationInvitation");
		idAction.add("deleteAttachmentFromCard");
		idAction.add("deleteBoardInvitation");
		idAction.add("deleteCard");
		idAction.add("deleteCheckItem");
		idAction.add("deleteLabel");
		idAction.add("deleteOrganizationInvitation");
		idAction.add("disableEnterprisePluginWhitelist");
		idAction.add("disablePlugin");
		idAction.add("disablePowerUp");
		idAction.add("emailCard");
		idAction.add("enableEnterprisePluginWhitelist");
		idAction.add("enablePlugin");
		idAction.add("enablePowerUp");
		idAction.add("makeAdminOfBoard");
		idAction.add("makeAdminOfOrganization");
		idAction.add("makeNormalMemberOfBoard");
		idAction.add("makeNormalMemberOfOrganization");
		idAction.add("makeObserverOfBoard");
		idAction.add("memberJoinedTrello");
		idAction.add("moveCardFromBoard");
		idAction.add("moveCardToBoard");
		idAction.add("moveListFromBoard");
		idAction.add("moveListToBoard");
		idAction.add("removeChecklistFromCard");
		idAction.add("removeFromEnterprisePluginWhitelist");
		idAction.add("removeFromOrganizationBoard");
		idAction.add("removeLabelFromCard");
		idAction.add("removeMemberFromBoard");
		idAction.add("removeMemberFromCard");
		idAction.add("removeMemberFromOrganization");
		idAction.add("removeOrganizationFromEnterprise");
		idAction.add("unconfirmedBoardInvitation");
		idAction.add("unconfirmedOrganizationInvitation");
		idAction.add("updateBoard");
		idAction.add("updateCard");
		idAction.add("updateCheckItem");
		idAction.add("updateCheckItemStateOnCard");
		idAction.add("updateChecklist");
		idAction.add("updateLabel");
		idAction.add("updateList");
		idAction.add("updateMember");
		idAction.add("updateOrganization");
		idAction.add("voteOnCard");
		return idAction;
	}
	
	private List<String> GetCardIds()
	{
		List<String> idCard = new ArrayList<>();
		idCard.add("1-pao");
		idCard.add("2-pao");
		idCard.add("3-pao");
		idCard.add("4-pao");
		return idCard;
	}
}
