import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.parser.OpenAPIResolver;
import io.swagger.v3.parser.OpenAPIV3Parser;
import pt.uc.dei.rest_api_robustness_tester.specification.*;
import pt.uc.dei.rest_api_robustness_tester.request.Parameter;
import pt.uc.dei.rest_api_robustness_tester.Path;
import pt.uc.dei.rest_api_robustness_tester.workload.ParameterFilter;

import java.io.File;
import java.util.List;
import java.util.ArrayList;

public class HighwaysEnglandAPI implements RestApiSetup
{
    @Override
    public RestApi Load()
    {
        File file = new File("API Specifications\\Public tier 2\\Highways England", "Highways England API.yaml");
        OpenAPI openAPI = new OpenAPIV3Parser().read(file.getAbsolutePath());
        OpenAPIResolver res = new OpenAPIResolver(openAPI);
        openAPI = res.resolve();
    
        RestApiConverter<OpenAPI> converter = new OpenApiToRestApi();
        RestApiSpecification restAPISpecification = converter.Convert(openAPI);
    
		List<String> areaIDs = new ArrayList<>();
		for(int i = 0; i < 25; i++)
			areaIDs.add("" + (i+1));
		
		List<String> reportTypes = new ArrayList<>();
		reportTypes.add("Daily");
		reportTypes.add("Monthly");
		reportTypes.add("Annual");
		
		List<String> siteIDs = new ArrayList<>();
		for(int i = 0; i < 17911; i++)
			siteIDs.add("" + (i+1));
		
		List<String> siteTypeIDs = new ArrayList<>();
		for(int i = 0; i < 4; i++)
			siteTypeIDs.add("" + (i+1));
	
		RestApiConfig config = new RestApiConfig();
        config.AddParameterFilter(new ParameterFilter("version", Parameter.Location.Path).SetValue("1.0").ApplyToAll()).
				AddParameterFilter(new ParameterFilter("area_Ids", Parameter.Location.Path).SetValues(areaIDs).ApplyToAll()).
				AddParameterFilter(new ParameterFilter("report_type", Parameter.Location.Path).SetValues(reportTypes).ApplyToAll()).
				AddParameterFilter(new ParameterFilter("site_Ids", Parameter.Location.Path).SetValues(siteIDs).ApplyToAll()).
				AddParameterFilter(new ParameterFilter("siteType_Id", Parameter.Location.Path).SetValues(siteTypeIDs).ApplyToAll());
	
        return new RestApi("Highways England API", restAPISpecification, config);
    }
}
