import io.swagger.v3.oas.models.OpenAPI;
import io.swagger.v3.parser.OpenAPIResolver;
import io.swagger.v3.parser.OpenAPIV3Parser;
import pt.uc.dei.rest_api_robustness_tester.security.AuthHandlers;
import pt.uc.dei.rest_api_robustness_tester.specification.*;
import pt.uc.dei.rest_api_robustness_tester.request.Parameter;
import pt.uc.dei.rest_api_robustness_tester.Path;
import pt.uc.dei.rest_api_robustness_tester.workload.OperationFilter;
import pt.uc.dei.rest_api_robustness_tester.workload.ParameterFilter;

import java.io.File;
import java.util.List;
import java.util.ArrayList;

public class LaunchDarkly implements RestApiSetup
{
    @Override
    public RestApi Load()
    {
        //New API Key can be obtained at https://app.launchdarkly.com/settings/authorization
        String apiKey = "api-d86807c8-6824-4bb8-9c98-da3609868ed6";
    
        File file = new File("API Specifications\\Public tier 2\\LaunchDarkly", "LaunchDarkly.yaml");
        AuthHandlers authHandlers = new AuthHandlers();
        authHandlers.AddAuthHandler("Token", requestBuilder -> requestBuilder.addHeader("Authorization", apiKey));
    
        OpenAPI openAPI = new OpenAPIV3Parser().read(file.getAbsolutePath());
        OpenAPIResolver res = new OpenAPIResolver(openAPI);
        openAPI = res.resolve();
    
        RestApiConverter<OpenAPI> converter = new OpenApiToRestApi();
        RestApiSpecification restAPISpecification = converter.Convert(openAPI);
    
		List<String> projectKeys = new ArrayList<>();
		projectKeys.add("default");
		projectKeys.add("proj-1");
		
		List<String> environmentKeys = new ArrayList<>();
		environmentKeys.add("production");
		environmentKeys.add("test");
		
		List<String> userSegmentKeys = new ArrayList<>();
		userSegmentKeys.add("segment-1");
		userSegmentKeys.add("segment-2");
		
		List<String> featureFlagKeys = new ArrayList<>();
		featureFlagKeys.add("feat-1");
		featureFlagKeys.add("feat-2");
		featureFlagKeys.add("feat-3");
		
		RestApiConfig config = new RestApiConfig();
        config.AddParameterFilter(new ParameterFilter("projectKey", Parameter.Location.Path).SetValues(projectKeys).ApplyToAll()).
				AddParameterFilter(new ParameterFilter("environmentKey", Parameter.Location.Path).SetValues(environmentKeys).ApplyToAll()).
				AddParameterFilter(new ParameterFilter("userSegmentKey", Parameter.Location.Path).SetValues(userSegmentKeys).ApplyToAll()).
				AddParameterFilter(new ParameterFilter("featureFlagKey", Parameter.Location.Path).SetValues(featureFlagKeys).ApplyToAll());
	
        return new RestApi("LaunchDarkly", restAPISpecification, authHandlers, config);
    }
}
