package pt.uc.dei.rest_api_robustness_tester.schema;

import org.apache.commons.lang3.RandomUtils;

import java.util.Random;

public class DoubleSchema extends NumericalSchema<Double>
{
    DoubleSchema(SchemaBuilder builder)
    {
        super(builder);
        try
        {
            if(builder.maximum != null)
                this.maximum = Double.parseDouble(builder.maximum);
            if(builder.minimum != null)
                this.minimum = Double.parseDouble(builder.minimum);
            if(builder.multipleOf != null)
                this.multipleOf = Double.parseDouble(builder.multipleOf);
            if (builder.defaultValue != null)
                this.defaultValue = Double.parseDouble(builder.defaultValue);
            if(!builder.enumValues.isEmpty())
                for(String s : builder.enumValues)
                    this.enumValues.add(Double.parseDouble(s));
        }
        catch(Exception ignored) {}
    }
    
    @Override
    protected Schema<Double> New() throws Exception
    {
        try
        {
            if (defaultValue != null && RandomUtils.nextFloat(0, 1) <= DEFAULT_PROBABILITY)
            {
                double avg = defaultValue;
                double std = avg / 2;
                value = new Random().nextGaussian() * std + avg;
                return this;
            }
        }
        catch(Exception ignored) {}
    
        if(enumValues.isEmpty())
        {
            Double min = minimum;
            Double max = maximum;
        
            min = min == Double.MIN_VALUE? min + 1 : min;
        
            if(min < 0)
                value = RandomUtils.nextDouble(0, max) - RandomUtils.nextDouble(0, Math.abs(min));
            else
                value = RandomUtils.nextDouble(min, max);
        }
        else
            value = enumValues.get(RandomUtils.nextInt(0, enumValues.size()));
        return this;
    }

//    @Override
//    protected String New(String value, MediaTypeFormatter formatter) throws Exception
//    {
//        return value;
//    }
    
    @Override
    protected Double DefaultMinimum()
    {
        return Double.MIN_VALUE;
    }
    
    @Override
    protected Double DefaultMaximum()
    {
        return Double.MAX_VALUE;
    }
}