package pt.uc.dei.rest_api_robustness_tester.schema;

import org.apache.commons.lang3.RandomUtils;

import java.util.Random;

public class FloatSchema extends NumericalSchema<Float>
{
    FloatSchema(SchemaBuilder builder)
    {
        super(builder);
        try
        {
            if(builder.maximum != null)
                this.maximum = Float.parseFloat(builder.maximum);
            if(builder.minimum != null)
                this.minimum = Float.parseFloat(builder.minimum);
            if(builder.multipleOf != null)
                this.multipleOf = Float.parseFloat(builder.multipleOf);
            if (builder.defaultValue != null)
                this.defaultValue = Float.parseFloat(builder.defaultValue);
            if(!builder.enumValues.isEmpty())
                for(String s : builder.enumValues)
                    this.enumValues.add(Float.parseFloat(s));
        }
        catch(Exception ignored) {}
    }
    
    @Override
    protected Schema<Float> New() throws Exception
    {
        try
        {
            if (defaultValue != null && RandomUtils.nextFloat(0, 1) <= DEFAULT_PROBABILITY)
            {
                float avg = defaultValue;
                float std = avg / 2;
                value = (float) (new Random().nextGaussian() * std + avg);
                return this;
            }
        }
        catch(Exception ignored) {}
    
        if(enumValues.isEmpty())
        {
            Float min = minimum;
            Float max = maximum;
        
            min = min == Float.MIN_VALUE? min + 1 : min;
        
            if(min < 0)
                value = RandomUtils.nextFloat(0, max) - RandomUtils.nextFloat(0, Math.abs(min));
            else
                value = RandomUtils.nextFloat(min, max);
        }
        else
            value = enumValues.get(RandomUtils.nextInt(0, enumValues.size()));
        return this;
    }

//    @Override
//    protected String New(String value, MediaTypeFormatter formatter) throws Exception
//    {
//        return value;
//    }
    
    @Override
    protected Float DefaultMinimum()
    {
        return Float.MIN_VALUE;
    }
    
    @Override
    protected Float DefaultMaximum()
    {
        return Float.MAX_VALUE;
    }
}