package pt.uc.dei.rest_api_robustness_tester.schema;

import org.apache.commons.lang3.RandomUtils;

import java.util.Random;

public class IntegerSchema extends NumericalSchema<Integer>
{
    IntegerSchema(SchemaBuilder builder)
    {
        super(builder);
        try
        {
            if(builder.maximum != null)
                this.maximum = Integer.parseInt(builder.maximum);
            if(builder.minimum != null)
                this.minimum = Integer.parseInt(builder.minimum);
            if(builder.multipleOf != null)
                this.multipleOf = Integer.parseInt(builder.multipleOf);
            if (builder.defaultValue != null)
                this.defaultValue = Integer.parseInt(builder.defaultValue);
            if(!builder.enumValues.isEmpty())
                for(String s : builder.enumValues)
                    this.enumValues.add(Integer.parseInt(s));
        }
        catch(Exception ignored) {}
    }
    
    @Override
    protected Schema<Integer> New() throws Exception
    {
        try
        {
            if (defaultValue != null && RandomUtils.nextFloat(0, 1) <= DEFAULT_PROBABILITY)
            {
                int avg = defaultValue;
                int std = avg / 2;
                value = (int) (new Random().nextGaussian() * std + avg);
                return this;
            }
        }
        catch(Exception ignored) {}
    
        if(enumValues.isEmpty())
        {
            Integer min = minimum;
            Integer max = maximum;
        
            max = max == Integer.MAX_VALUE? max : max + 1;
            min = min == Integer.MIN_VALUE? min + 1 : min;
        
            if(min < 0)
                value = RandomUtils.nextInt(0, max) - RandomUtils.nextInt(0, Math.abs(min));
            else
                value = RandomUtils.nextInt(min, max);
        }
        else
            value = enumValues.get(RandomUtils.nextInt(0, enumValues.size()));
        return this;
    }
    
//    @Override
//    protected String New(String value, MediaTypeFormatter formatter) throws Exception
//    {
//        return value;
//    }
    
    @Override
    protected Integer DefaultMinimum()
    {
        return Integer.MIN_VALUE;
    }
    
    @Override
    protected Integer DefaultMaximum()
    {
        return Integer.MAX_VALUE;
    }
}