package pt.uc.dei.rest_api_robustness_tester.schema;

import org.apache.commons.lang3.RandomUtils;

import java.util.Random;

public class LongSchema extends NumericalSchema<Long>
{
    LongSchema(SchemaBuilder builder)
    {
        super(builder);
        try
        {
            if(builder.maximum != null)
                this.maximum = Long.parseLong(builder.maximum);
            if(builder.minimum != null)
                this.minimum = Long.parseLong(builder.minimum);
            if(builder.multipleOf != null)
                this.multipleOf = Long.parseLong(builder.multipleOf);
            if (builder.defaultValue != null)
                this.defaultValue = Long.parseLong(builder.defaultValue);
            if(!builder.enumValues.isEmpty())
                for(String s : builder.enumValues)
                    this.enumValues.add(Long.parseLong(s));
        }
        catch(Exception ignored) {}
    }
    
    @Override
    protected Schema<Long> New() throws Exception
    {
        try
        {
            if (defaultValue != null && RandomUtils.nextFloat(0, 1) <= DEFAULT_PROBABILITY)
            {
                long avg = defaultValue;
                long std = avg / 2;
                value = (long) (new Random().nextGaussian() * std + avg);
                return this;
            }
        }
        catch(Exception ignored) {}
    
        if(enumValues.isEmpty())
        {
            Long min = minimum;
            Long max = maximum;
        
            max = max == Long.MAX_VALUE? max : max + 1;
            min = min == Long.MIN_VALUE? min + 1 : min;
        
            if(min < 0)
                value = RandomUtils.nextLong(0, max) - RandomUtils.nextLong(0, Math.abs(min));
            else
                value = RandomUtils.nextLong(min, max);
        }
        else
            value = enumValues.get(RandomUtils.nextInt(0, enumValues.size()));
        return this;
    }
    
//    @Override
//    protected String New(String value, MediaTypeFormatter formatter) throws Exception
//    {
//        return value;
//    }
    
    @Override
    protected Long DefaultMinimum()
    {
        return Long.MIN_VALUE;
    }
    
    @Override
    protected Long DefaultMaximum()
    {
        return Long.MAX_VALUE;
    }
}