package pt.uc.dei.rest_api_robustness_tester.schema;

import pt.uc.dei.rest_api_robustness_tester.media.FormatterManager;
import pt.uc.dei.rest_api_robustness_tester.media.SchemaFormatter;

public abstract class Schema<T>
{
    //TODO: should be Global constant
    protected static final float DEFAULT_PROBABILITY = 0.25f;
    
    //TODO: add support for XML property
    public boolean nullable = false;
    public boolean readOnly = false;
    public boolean writeOnly = false;
    
    public T value = null;
    
    public String type = null;
    public String format = null;
    
    Schema(SchemaBuilder builder)
    {
        this.nullable = builder.nullable;
        this.readOnly = builder.readOnly;
        this.writeOnly = builder.writeOnly;
        this.type = builder.type;
        this.format = builder.format;
    }
    
    public SchemaInstance Instantiate(String value, String mediaType) throws Exception
    {
        if(value != null)
            //this.value = New(value);
            return new SchemaInstance(value, this);
        else
            New();
        
        Class<? extends Schema> c;
        if(TypeManager.Instance().HasFormat(format))
            c = TypeManager.Instance().GetFormat(format);
        else
            c = TypeManager.Instance().GetType(type);
    
        SchemaFormatter f;
        if(mediaType != null)
            f = FormatterManager.Instance().GetFormatter(mediaType, c);
        else
            f = FormatterManager.Instance().GetDefaultFormatter();
        return new SchemaInstance(f.Serialize(this), this);
    }
    
    protected abstract Schema<T> New() throws Exception;
    //TODO: instead of deserializing the *value* parameter, should just check if
    //      parameter's content agrees with the Schema's data type
    //      (e.g., for an ObjectSchema, should check if content is structured as a map and
    //      for each of its elements, check if its value also agrees with its respective schema)
    //      (the latter part of the e.g. probably depends on deserialization, though)
    //protected abstract T New(String value) throws Exception;
    
    @Override
    public String toString()
    {
        return value.toString();
    }
    
    public static SchemaBuilder Builder()
    {
        return new SchemaBuilder();
    }
}
