package pt.uc.dei.rest_api_robustness_tester.schema;

import com.github.curiousoddman.rgxgen.RgxGen;
import com.mifmif.common.regex.Generex;
import org.apache.commons.lang3.RandomUtils;
import pt.uc.dei.rest_api_robustness_tester.utils.Utils;

public class StringSchema extends PrimitiveSchema<String>
{
    //TODO: maybe by default use String.MIN and String.MAX?
    //      or
    //      some MIN and MAX Global constants?
    public Integer minLength = 1;
    public Integer maxLength = Utils.AsciiAlphanumericCharacters().length() - 1;
    
    public String pattern = null;
    
    StringSchema(SchemaBuilder builder)
    {
        super(builder);
        try
        {
            if(builder.maximum != null)
                this.maxLength = Integer.parseInt(builder.maximum);
            if(builder.minimum != null)
                this.minLength = Integer.parseInt(builder.minimum);
            if(builder.pattern != null)
                this.pattern = builder.pattern;
            if (builder.defaultValue != null)
                this.defaultValue = builder.defaultValue;
            if(!builder.enumValues.isEmpty())
                this.enumValues.addAll(builder.enumValues);
        }
        catch(Exception ignored) {}
    }
    
    @Override
    protected Schema<String> New() throws Exception
    {
        //TODO: some APIs represent booleans or numbers as Strings, and the default value (if provided)
        //      is the only way of telling that
        //      Therefore, it should be useful to infer the Schema's real type from its default value
        //      and generate a new value accordingly
        if(defaultValue != null && RandomUtils.nextFloat(0, 1) <= DEFAULT_PROBABILITY)
        {
            value = defaultValue;
            return this;
        }
    
        if(pattern != null)
        {
            try
            {
                value = new RgxGen(pattern).generate();
            }
            catch(Exception e)
            {
                System.err.println("Failed to generate string using RgxGen library (pattern: " + pattern +")");
                value = new Generex(pattern).random();
            }
            
            return this;
        }
        
        if(enumValues.isEmpty())
        {
            Integer min = minLength;
            Integer max = maxLength;
        
            max = max == Integer.MAX_VALUE? max : max + 1;
        
            StringBuilder s = new StringBuilder();
        
            for(int i = 0; i < RandomUtils.nextInt(min, max); i++)
                s.append(Utils.AsciiAlphanumericCharacters().
                        charAt(RandomUtils.nextInt(0, Utils.AsciiAlphanumericCharacters().length())));
    
            value = s.toString();
        }
        else
            value = enumValues.get(RandomUtils.nextInt(0, enumValues.size()));
        return this;
    }
    
//    @Override
//    protected String New(String value, MediaTypeFormatter formatter) throws Exception
//    {
//        return value;
//    }
}