package pt.uc.dei.rest_api_robustness_tester.utils;

import java.util.Map;

public final class Config
{
    public static final int UNKNOWN_PORT = -1;
    
    private static Config instance = null;
    
    public static Config Instance() {return instance;}
    
    public static boolean IsInitialized()
    {
        return instance != null;
    }
    
    public Config(Map<String, String> args) throws Exception
    {
        apiClassFile = args.get("--api-file");
        workloadResultsFile = args.get("--wl-results");
        faultloadResultsFile = args.get("--fl-results");

        WL_REP = Integer.parseInt(args.getOrDefault("--wl-rep", "10"));
        WL_RETRY = Integer.parseInt(args.getOrDefault("--wl-retry", "1"));
        WL_MAX_TIME = Integer.parseInt(args.getOrDefault("--wl-max-time", "0"));
        FL_REP = Integer.parseInt(args.getOrDefault("--fl-rep", "3"));
        FL_MAX_TIME = Integer.parseInt(args.getOrDefault("--fl-max-time", "0"));

        out = args.get("--out");
        keepFailedWorkload = args.containsKey("--keep-failed-wl");
        proxyPort = Integer.parseInt(args.getOrDefault("--proxy-port", "" + UNKNOWN_PORT));
        connTimeout = Integer.parseInt(args.getOrDefault("--conn-timeout", "" + 60));
        
        if(apiClassFile == null)
            throw new Exception("Parameter --api-file is mandatory");
        if(workloadResultsFile == null)
            throw new Exception("Parameter --wl-results is mandatory");
        if(faultloadResultsFile == null)
            throw new Exception("Parameter --fl-results is mandatory");
        
        if(!apiClassFile.endsWith(".java"))
            throw new Exception("Value for parameter --api-file must follow the pattern *.java");
    
        if(!workloadResultsFile.endsWith(".xlsx"))
            throw new Exception("Value for parameter --wl-results must follow the pattern *.xlsx");
        if(!faultloadResultsFile.endsWith(".xlsx"))
            throw new Exception("Value for parameter --fl-results must follow the pattern *.xlsx");
        
        instance = this;
    }
    
    public final String apiClassFile;
    public final String workloadResultsFile;
    public final String faultloadResultsFile;
    
    public final int WL_REP;
    public final int WL_RETRY;
    public final int WL_MAX_TIME;
    public final int FL_REP;
    public final int FL_MAX_TIME;
    public final String out;
    public final boolean keepFailedWorkload;
    public final int proxyPort;
    public final int connTimeout;
}
